@@ -1,10 +1,10 @@
 /*
  *
  * $Id.cpp,v 1.82 2005/02/04 09:27:19 trueg Exp $
- * Copyright (C) 2003-2004 Sebastian Trueg <trueg@k3b.org>
+ * Copyright (C) 2003-2007 Sebastian Trueg <trueg@k3b.org>
  *
  * This file is part of the K3b project.
- * Copyright (C) 1998-2004 Sebastian Trueg <trueg@k3b.org>
+ * Copyright (C) 1998-2007 Sebastian Trueg <trueg@k3b.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -41,7 +41,6 @@
 #include <kio/netaccess.h>
 #include <kio/job.h>
 #include <kio/global.h>
-#include <kmessagebox.h>
 
 #include <qtimer.h>
 #include <qstringlist.h>
@@ -122,9 +121,11 @@ K3bCdCopyJob::K3bCdCopyJob( K3bJobHandler* hdl, QObject* parent )
     m_copies(1),
     m_onlyCreateImages(false),
     m_onTheFly(true),
-    m_ignoreReadErrors(false),
+    m_ignoreDataReadErrors(false),
+    m_ignoreAudioReadErrors(true),
     m_noCorrection(false),
-    m_readRetries(128),
+    m_dataReadRetries(128),
+    m_audioReadRetries(5),
     m_preferCdText(false),
     m_copyCdText(true),
     m_writingMode( K3b::WRITING_MODE_AUTO )
@@ -150,15 +151,15 @@ void K3bCdCopyJob::start()
   d->sessionSizes.clear();
   d->dataSessionProbablyTAORecorded.clear();
   d->deleteTempDir = false;
+  d->haveCdText = false;
+  d->haveCddb = false;
 
-  emit started();
+  jobStarted();
 
-  emit newTask( i18n("Checking Source Disk") );
-
-  // without this message the window looks quite empty and the user might think nothing is going on
-  emit infoMessage( i18n("Checking source disk"), INFO );
+  emit newTask( i18n("Checking Source Medium") );
 
   emit burning(false);
+  emit newSubTask( i18n("Waiting for source medium") );
 
   // wait for a source disk
   if( waitForMedia( m_readerDevice,
@@ -168,7 +169,7 @@ void K3bCdCopyJob::start()
     return;
   }
 
-  emit newSubTask( i18n("Reading Table of Contents") );
+  emit newSubTask( i18n("Checking source medium") );
 
   // FIXME: read ISRCs and MCN
 
@@ -184,7 +185,7 @@ void K3bCdCopyJob::slotDiskInfoReady( K3bDevice::DeviceHandler* dh )
 
     //
     // for now we copy audio, pure data (aka 1 data track), cd-extra (2 session, audio and data),
-    // and data multisession wich one track per session.
+    // and data multisession which one track per session.
     // Everything else will be rejected
     //
     bool canCopy = true;
@@ -326,6 +327,11 @@ void K3bCdCopyJob::slotDiskInfoReady( K3bDevice::DeviceHandler* dh )
     }
 
     if( canCopy ) {
+      if( K3b::isMounted( m_readerDevice ) ) {
+	emit infoMessage( i18n("Unmounting source medium"), INFO );
+	K3b::unmount( m_readerDevice );
+      }
+
       d->overallSize = 0;
 
       // now create some progress helper values
@@ -548,10 +554,8 @@ bool K3bCdCopyJob::prepareImageFiles()
   else {
     // we only need a single image file
     if( !fi.isFile() || 
-	KMessageBox::warningYesNo( qApp->activeWindow(),
-				   i18n("Do you want to overwrite %1?").arg(m_tempPath),
-				   i18n("File Exists") )
-	== KMessageBox::Yes) {
+	questionYesNo( i18n("Do you want to overwrite %1?").arg(m_tempPath),
+		       i18n("File Exists") ) ) {
       if( fi.isDir() )
 	m_tempPath = K3b::findTempFile( "iso", m_tempPath );
       else if( !QFileInfo( m_tempPath.section( '/', 0, -2 ) ).isDir() ) {
@@ -578,7 +582,7 @@ void K3bCdCopyJob::readNextSession()
     if( d->numSessions > 1 )
       emit newTask( i18n("Reading Session %1").arg(d->currentReadSession) );
     else
-      emit newTask( i18n("Reading Source Disk") );
+      emit newTask( i18n("Reading Source Medium") );
 
     if( d->currentReadSession == 1 )
       emit newSubTask( i18n("Reading track %1 of %2").arg(1).arg(d->toc.count()) );
@@ -602,8 +606,8 @@ void K3bCdCopyJob::readNextSession()
     d->audioSessionReader->setDevice( m_readerDevice );
     d->audioSessionReader->setToc( d->toc );
     d->audioSessionReader->setParanoiaMode( m_paranoiaMode );
-    d->audioSessionReader->setReadRetries( m_readRetries );
-    d->audioSessionReader->setNeverSkip( !m_ignoreReadErrors );
+    d->audioSessionReader->setReadRetries( m_audioReadRetries );
+    d->audioSessionReader->setNeverSkip( !m_ignoreAudioReadErrors );
     if( m_onTheFly )
       d->audioSessionReader->writeToFd( d->cdrecordWriter->fd() );
     else
@@ -624,9 +628,9 @@ void K3bCdCopyJob::readNextSession()
     }
 
     d->dataTrackReader->setDevice( m_readerDevice );
-    d->dataTrackReader->setIgnoreErrors( m_ignoreReadErrors );
+    d->dataTrackReader->setIgnoreErrors( m_ignoreDataReadErrors );
     d->dataTrackReader->setNoCorrection( m_noCorrection );
-    d->dataTrackReader->setRetries( m_readRetries );
+    d->dataTrackReader->setRetries( m_dataReadRetries );
     if( m_onlyCreateImages )
       d->dataTrackReader->setSectorSize( K3bDataTrackReader::MODE1 );
     else
@@ -662,6 +666,7 @@ void K3bCdCopyJob::readNextSession()
     d->dataReaderRunning = true;
     if( !m_onTheFly || m_onlyCreateImages )
       slotReadingNextTrack( 1, 1 );
+
     d->dataTrackReader->start();
   }
 }
@@ -874,8 +879,9 @@ bool K3bCdCopyJob::writeNextSession()
     bool multi = d->doNotCloseLastSession || (d->numSessions > 1 && d->currentWrittenSession < d->toc.count());
     int usedWritingMode = m_writingMode;
     if( usedWritingMode == K3b::WRITING_MODE_AUTO ) {
-      // use DAO in overwriting mode when not writing multi-session or multi-track
-      k3bcore->config()->setGroup("General Options");
+      // at least the NEC3540a does write 2056 byte sectors only in tao mode. Same for LG4040b
+      // since writing data tracks in TAO mode is no loss let's default to TAO in the case of 2056 byte
+      // sectors (which is when writing xa form1 sectors here)
       if( m_writerDevice->dao() &&
 	  d->toc.count() == 1 && 
 	  !multi &&
@@ -923,6 +929,9 @@ bool K3bCdCopyJob::writeNextSession()
       d->cdrecordWriter->addArgument( d->imageNames[d->toc.count()-1] );
     else
       d->cdrecordWriter->addArgument( d->imageNames[d->currentWrittenSession-1] );
+
+    // clear cd text from previous sessions
+    d->cdrecordWriter->setRawCdText( QByteArray() );
   }
 
 
@@ -961,9 +970,7 @@ void K3bCdCopyJob::slotSessionReaderFinished( bool success )
 	    // becasue if it did not it might happen that k3b overwrites a CD-RW
 	    // source)
 	    if( !m_readerDevice->eject() ) {
-	      emit infoMessage( i18n("Unable to eject media."), ERROR );
-	      finishJob( false, true );
-	      return;
+	      blockingInformation( i18n("K3b was unable to eject the source disk. Please do so manually.") );
 	    }
 	  }
 	  
@@ -1007,7 +1014,7 @@ void K3bCdCopyJob::slotWriterFinished( bool success )
       d->currentReadSession++;
 
       // reload the media
-      emit newSubTask( i18n("Reloading the media") );
+      emit newSubTask( i18n("Reloading the medium") );
       connect( K3bDevice::reload( m_writerDevice ), SIGNAL(finished(K3bDevice::DeviceHandler*)),
 	       this, SLOT(slotMediaReloadedForNextSession(K3bDevice::DeviceHandler*)) );
     }
@@ -1050,8 +1057,8 @@ void K3bCdCopyJob::slotWriterFinished( bool success )
 void K3bCdCopyJob::slotMediaReloadedForNextSession( K3bDevice::DeviceHandler* dh )
 {
   if( !dh->success() )
-    KMessageBox::information( qApp->activeWindow(), i18n("Please reload the medium and press 'ok'"),
-			      i18n("Unable to close the tray") );
+    blockingInformation( i18n("Please reload the medium and press 'ok'"),
+			 i18n("Unable to close the tray") );
 
   if( !writeNextSession() ) {
     // nothing is running here...
@@ -1077,7 +1084,7 @@ void K3bCdCopyJob::cleanup()
 
     // remove the tempdir created in prepareImageFiles()
     if( d->deleteTempDir ) {
-      KIO::NetAccess::del( KURL::fromPathOrURL(m_tempPath) );
+      KIO::NetAccess::del( KURL::fromPathOrURL(m_tempPath), 0 );
       d->deleteTempDir = false;
     }
   }
@@ -1199,7 +1206,7 @@ void K3bCdCopyJob::finishJob( bool c, bool e )
     
     d->running = false;
     
-    emit finished( !(c||e) );
+    jobFinished( !(c||e) );
   }
 }
 
