@@ -1,10 +1,10 @@
 /* 
  *
- * $Id: k3bdiritem.cpp 550094 2006-06-11 00:10:49Z trueg $
+ * $Id: k3bdiritem.cpp 619556 2007-01-03 17:38:12Z trueg $
  * Copyright (C) 2003 Sebastian Trueg <trueg@k3b.org>
  *
  * This file is part of the K3b project.
- * Copyright (C) 1998-2004 Sebastian Trueg <trueg@k3b.org>
+ * Copyright (C) 1998-2007 Sebastian Trueg <trueg@k3b.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -28,6 +28,9 @@
 K3bDirItem::K3bDirItem(const QString& name, K3bDataDoc* doc, K3bDirItem* parentDir)
   : K3bDataItem( doc, parentDir ),
     m_size(0),
+    m_followSymlinksSize(0),
+    m_blocks(0),
+    m_followSymlinksBlocks(0),
     m_files(0),
     m_dirs(0)
 {
@@ -38,6 +41,21 @@ K3bDirItem::K3bDirItem(const QString& name, K3bDataDoc* doc, K3bDirItem* parentD
     parent()->addDataItem( this );
 }
 
+
+K3bDirItem::K3bDirItem( const K3bDirItem& item )
+  : K3bDataItem( item ),
+    m_size(0),
+    m_followSymlinksSize(0),
+    m_blocks(0),
+    m_followSymlinksBlocks(0),
+    m_files(0),
+    m_dirs(0),
+    m_localPath( item.m_localPath )
+{
+  for( QPtrListIterator<K3bDataItem> it( item.children() ); *it; ++it )
+    addDataItem( (*it)->copy() );
+}
+
 K3bDirItem::~K3bDirItem()
 {
   // delete all children
@@ -59,6 +77,13 @@ K3bDirItem::~K3bDirItem()
   take();
 }
 
+
+K3bDataItem* K3bDirItem::copy() const
+{
+  return new K3bDirItem( *this );
+}
+
+
 K3bDirItem* K3bDirItem::getDirItem() const
 {
   return const_cast<K3bDirItem*>(this);
@@ -79,7 +104,7 @@ K3bDirItem* K3bDirItem::addDataItem( K3bDataItem* item )
       // do we replace an old item?
       QString name = item->k3bName();
       int cnt = 1;
-      while( K3bDataItem* oldItem = find( item->k3bName() ) ) {
+      while( K3bDataItem* oldItem = find( name ) ) {
 	if( !oldItem->isDir() && oldItem->isFromOldSession() ) {
 	  // in this case we remove this item from it's parent and save it in the new one
 	  // to be able to recover it
@@ -88,7 +113,7 @@ K3bDirItem* K3bDirItem::addDataItem( K3bDataItem* item )
 	  static_cast<K3bFileItem*>(item)->setReplacedItemFromOldSession( oldItem );
 	  break;
 	}
-      	else {
+	else {
 	  //
 	  // add a counter to the filename
 	  //
@@ -102,7 +127,7 @@ K3bDirItem* K3bDirItem::addDataItem( K3bDataItem* item )
     }
 
     m_children.append( item->take() );
-    updateSize( item->size() );
+    updateSize( item, false );
     if( item->isDir() )
       updateFiles( ((K3bDirItem*)item)->numFiles(), ((K3bDirItem*)item)->numDirs()+1 );
     else
@@ -111,7 +136,8 @@ K3bDirItem* K3bDirItem::addDataItem( K3bDataItem* item )
     item->m_parentDir = this;
 
     // inform the doc
-    doc()->itemAddedToDir( this, item );
+    if( doc() )
+      doc()->itemAddedToDir( this, item );
   }
 
   return this;
@@ -123,7 +149,7 @@ K3bDataItem* K3bDirItem::takeDataItem( K3bDataItem* item )
   int x = m_children.findRef( item );
   if( x > -1 ) {
     K3bDataItem* item = m_children.take();
-    updateSize( -1*item->size() );
+    updateSize( item, true );
     if( item->isDir() )
       updateFiles( -1*((K3bDirItem*)item)->numFiles(), -1*((K3bDirItem*)item)->numDirs()-1 );
     else
@@ -132,7 +158,8 @@ K3bDataItem* K3bDirItem::takeDataItem( K3bDataItem* item )
     item->m_parentDir = 0;
     
     // inform the doc
-    doc()->itemRemovedFromDir( this, item );
+    if( doc() )
+      doc()->itemRemovedFromDir( this, item );
     
     if( item->isFile() ) {
       // restore the item imported from an old session
@@ -175,8 +202,7 @@ bool K3bDirItem::alreadyInDirectory( const QString& filename ) const
 
 K3bDataItem* K3bDirItem::find( const QString& filename ) const
 {
-  QPtrListIterator<K3bDataItem> it( m_children );
-  for( ; it.current(); ++it ) {
+  for( QPtrListIterator<K3bDataItem> it( m_children ); it.current(); ++it ) {
     if( it.current()->k3bName() == filename )
       return it.current();
   }
@@ -206,9 +232,60 @@ K3bDataItem* K3bDirItem::findByPath( const QString& p )
 }
 
 
-KIO::filesize_t K3bDirItem::size() const
+bool K3bDirItem::mkdir( const QString& dirPath )
 {
-  return m_size;
+  // 
+  // An absolut path always starts at the root item
+  //
+  if( dirPath[0] == '/' ) {
+    if( parent() )
+      return parent()->mkdir( dirPath );
+    else
+      return mkdir( dirPath.mid( 1 ) );
+  }
+
+  if( findByPath( dirPath ) )
+    return false;
+
+  QString restPath;
+  QString dirName;
+  int pos = dirPath.find( '/' );
+  if( pos == -1 ) {
+    dirName = dirPath;
+  }
+  else {
+    dirName = dirPath.left( pos );
+    restPath = dirPath.mid( pos+1 );
+  }
+
+  K3bDataItem* dir = find( dirName );
+  if( !dir )
+    dir = new K3bDirItem( dirName, doc(), this );
+  else if( !dir->isDir() )
+    return false;
+
+  if( !restPath.isEmpty() )
+    return static_cast<K3bDirItem*>(dir)->mkdir( restPath );
+
+  return true;
+}
+
+
+KIO::filesize_t K3bDirItem::itemSize( bool followsylinks ) const
+{
+  if( followsylinks )
+    return m_followSymlinksSize;
+  else
+    return m_size;
+}
+
+
+K3b::Msf K3bDirItem::itemBlocks( bool followSymlinks ) const
+{
+  if( followSymlinks )
+    return m_followSymlinksBlocks;
+  else
+    return m_blocks;
 }
 
 
@@ -232,21 +309,12 @@ bool K3bDirItem::isSubItem( K3bDataItem* item ) const
 long K3bDirItem::numFiles() const
 {
   return m_files;
-  int num = 0;
-
-  QPtrListIterator<K3bDataItem> it( m_children );
-  for( ; it.current(); ++it )
-    if( !it.current()->isDir() )
-      num++;
-
-  return num;
 }
 
 
 long K3bDirItem::numDirs() const
 {
   return m_dirs;
-  return m_children.count() - numFiles();
 }
 
 
@@ -255,19 +323,32 @@ bool K3bDirItem::isRemoveable() const
   if( !K3bDataItem::isRemoveable() )
     return false;
 
-  bool rem = true;
-  QPtrListIterator<K3bDataItem> it( m_children );
-  for( ; it.current(); ++it )
-    rem = rem && it.current()->isRemoveable();
-  return rem;
+  for( QPtrListIterator<K3bDataItem> it( m_children ); it.current(); ++it ) {
+    if( !it.current()->isRemoveable() )
+      return false;
+  }
+
+  return true;
 }
 
 
-void K3bDirItem::updateSize( KIO::filesize_t s )
+void K3bDirItem::updateSize( K3bDataItem* item, bool removed )
 {
-  m_size += s;
+  if( removed ) {
+    m_followSymlinksSize -= item->itemSize( true );
+    m_size -= item->itemSize( false );
+    m_followSymlinksBlocks -= item->itemBlocks( true ).lba();
+    m_blocks -= item->itemBlocks( false ).lba();
+  }
+  else {
+    m_followSymlinksSize += item->itemSize( true );
+    m_size += item->itemSize( false );
+    m_followSymlinksBlocks += item->itemBlocks( true ).lba();
+    m_blocks += item->itemBlocks( false ).lba();
+  }
+
   if( parent() )
-    parent()->updateSize( s );
+    parent()->updateSize( item, removed );
 }
 
 void K3bDirItem::updateFiles( long files, long dirs )
@@ -281,10 +362,10 @@ void K3bDirItem::updateFiles( long files, long dirs )
 
 bool K3bDirItem::isFromOldSession() const
 {
-  QPtrListIterator<K3bDataItem> it( m_children );
-  for( ; it.current(); ++it )
+  for( QPtrListIterator<K3bDataItem> it( m_children ); it.current(); ++it ) {
     if( (*it)->isFromOldSession() )
       return true;
+  }
   return false;
 }
 
@@ -292,15 +373,14 @@ bool K3bDirItem::isFromOldSession() const
 bool K3bDirItem::writeToCd() const
 {
   // check if this dir contains items to write
-  QPtrListIterator<K3bDataItem> it( m_children );
-  for( ; it.current(); ++it )
+  for( QPtrListIterator<K3bDataItem> it( m_children ); it.current(); ++it ) {
     if( (*it)->writeToCd() )
       return true;
+  }
   return K3bDataItem::writeToCd();
 }
 
 
-
 K3bRootItem::K3bRootItem( K3bDataDoc* doc )
   : K3bDirItem( "root", doc, 0 )
 {
@@ -312,7 +392,7 @@ K3bRootItem::~K3bRootItem()
 }
 
 
-const QString& K3bRootItem:: k3bName()
+const QString& K3bRootItem::k3bName() const
 {
   return doc()->isoOptions().volumeID();
 }
