@@ -1,10 +1,10 @@
 /*
  *
- * $Id: k3bisoimager.cpp 531385 2006-04-19 08:20:20Z trueg $
- * Copyright (C) 2003 Sebastian Trueg <trueg@k3b.org>
+ * $Id: k3bisoimager.cpp 633751 2007-02-15 08:22:49Z trueg $
+ * Copyright (C) 2003-2007 Sebastian Trueg <trueg@k3b.org>
  *
  * This file is part of the K3b project.
- * Copyright (C) 1998-2004 Sebastian Trueg <trueg@k3b.org>
+ * Copyright (C) 1998-2007 Sebastian Trueg <trueg@k3b.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -13,16 +13,22 @@
  * See the file "COPYING" for the exact licensing terms.
  */
 
+#include <config.h>
+#include <k3bglobals.h>
+
 #include "k3bisoimager.h"
 #include "k3bdiritem.h"
 #include "k3bbootitem.h"
 #include "k3bdatadoc.h"
+#include "k3bdatapreparationjob.h"
 #include <k3bexternalbinmanager.h>
 #include <k3bdevice.h>
 #include <k3bprocess.h>
 #include <k3bcore.h>
 #include <k3bversion.h>
 #include <k3bglobals.h>
+#include <k3bchecksumpipe.h>
+#include <k3bfilesplitter.h>
 
 #include <kdebug.h>
 #include <kstandarddirs.h>
@@ -31,29 +37,36 @@
 #include <kio/netaccess.h>
 #include <kio/global.h>
 #include <kio/job.h>
+#include <kstringhandler.h>
 
 #include <qfile.h>
 #include <qregexp.h>
-#include <qtimer.h>
 #include <qdir.h>
 #include <qapplication.h>
+#include <qvaluestack.h>
 
-#include <errno.h>
-#include <string.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <unistd.h>
 #include <utime.h>
 
 
-
 int K3bIsoImager::s_imagerSessionCounter = 0;
 
+
 class K3bIsoImager::Private
 {
 public:
+  Private()
+    : pipe(0) {
+  }
+
+  ~Private() {
+    delete pipe;
+  }
+
   QString imagePath;
-  QFile imageFile;
+  K3bFileSplitter imageFile;
   const K3bExternalBin* mkisofsBin;
 
   enum LinkHandling {
@@ -64,6 +77,11 @@ public:
   };
 
   int usedLinkHandling;
+
+  bool knownError;
+
+  K3bActivePipe* pipe;
+  K3bDataPreparationJob* dataPreparationJob;
 };
 
 
@@ -82,20 +100,24 @@ K3bIsoImager::K3bIsoImager( K3bDataDoc* doc, K3bJobHandler* hdl, QObject* parent
     m_mkisofsPrintSizeResult( 0 ),
     m_fdToWriteTo(-1)
 {
-  d = new Private;
+  d = new Private();
+  d->dataPreparationJob = new K3bDataPreparationJob( doc, this, this );
+  connectSubJob( d->dataPreparationJob,
+		 SLOT(slotDataPreparationDone(bool)),
+		 DEFAULT_SIGNAL_CONNECTION );
 }
 
 
 K3bIsoImager::~K3bIsoImager()
 {
-  delete d;
   cleanup();
+  delete d;
 }
 
 
 bool K3bIsoImager::active() const
 {
-  return !m_processExited;
+  return K3bJob::active();
 }
 
 
@@ -114,7 +136,7 @@ void K3bIsoImager::writeToImageFile( const QString& path )
 
 void K3bIsoImager::slotReceivedStderr( const QString& line )
 {
-  handleMkisofsOutput( line );
+  parseMkisofsOutput( line );
   emit debuggingOutput( "mkisofs", line );
 }
 
@@ -128,23 +150,42 @@ void K3bIsoImager::handleMkisofsProgress( int p )
 void K3bIsoImager::handleMkisofsInfoMessage( const QString& line, int type )
 {
   emit infoMessage( line, type );
+  if( type == ERROR )
+    d->knownError = true;
 }
 
 
 void K3bIsoImager::slotProcessExited( KProcess* p )
 {
+  kdDebug() << k_funcinfo << endl;
+
   m_processExited = true;
 
-  if( d->imageFile.isOpen() )
+  d->pipe->close();
+
+  emit debuggingOutput( "K3bIsoImager", 
+			QString("Pipe throughput: %1 bytes read, %2 bytes written.")
+			.arg(d->pipe->bytesRead()).arg(d->pipe->bytesWritten()) );
+
+  if( d->imageFile.isOpen() ) {
     d->imageFile.close();
 
-  if( !m_canceled ) {
+    if( m_canceled || p->exitStatus() != 0 ) {
+      d->imageFile.remove();
+      emit infoMessage( i18n("Removed incomplete image file %1.").arg(d->imageFile.name()), WARNING );
+    }
+  }
 
+  if( m_canceled ) {
+    emit canceled();
+    jobFinished(false);
+  }
+  else {
     if( p->normalExit() ) {
       if( p->exitStatus() == 0 ) {
-	emit finished( true );
+	jobFinished( !mkisofsReadError() );
       }
-      else  {
+      else {
 	switch( p->exitStatus() ) {
 	case 104:
 	  // connection reset by peer
@@ -159,7 +200,7 @@ void K3bIsoImager::slotProcessExited( KProcess* p )
 	  // mkisofs 1.14 has the bug, 1.15a40 not
 	  // TODO: find out the version that fixed the bug
 	  if( m_containsFilesWithMultibleBackslashes &&
-	      k3bcore->externalBinManager()->binObject( "mkisofs" )->version < K3bVersion( 1, 15, -1, "a40" ) ) {
+	      !k3bcore->externalBinManager()->binObject( "mkisofs" )->hasFeature( "backslashed_filenames" ) ) {
 	    emit infoMessage( i18n("Due to a bug in mkisofs <= 1.15a40, K3b is unable to handle "
 				   "filenames that contain more than one backslash:"), ERROR );
 
@@ -168,20 +209,19 @@ void K3bIsoImager::slotProcessExited( KProcess* p )
 	  // otherwise just fall through
 
 	default:
-	  if( !mkisofsReadError() ) {
+	  if( !d->knownError && !mkisofsReadError() ) {
 	    emit infoMessage( i18n("%1 returned an unknown error (code %2).").arg("mkisofs").arg(p->exitStatus()),
 			      K3bJob::ERROR );
-	    emit infoMessage( strerror(p->exitStatus()), K3bJob::ERROR );
 	    emit infoMessage( i18n("Please send me an email with the last output."), K3bJob::ERROR );
 	  }
 	}
 
-	emit finished( false );
+	jobFinished( false );
       }
     }
     else {
       emit infoMessage( i18n("%1 did not exit cleanly.").arg("mkisofs"), ERROR );
-      emit finished( false );
+      jobFinished( false );
     }
   }
 
@@ -212,41 +252,65 @@ void K3bIsoImager::cleanup()
 }
 
 
+void K3bIsoImager::init()
+{
+  jobStarted();
+
+  cleanup();
+
+  d->dataPreparationJob->start();
+}
+
+
+void K3bIsoImager::slotDataPreparationDone( bool success )
+{
+  if( success ) {
+    //
+    // We always calculate the image size. It does not take long and at least the mixed job needs it
+    // anyway
+    //
+    startSizeCalculation();
+  }
+  else {
+    if( d->dataPreparationJob->hasBeenCanceled() ) {
+      m_canceled = true;
+      emit canceled();
+    }
+    jobFinished( false );
+  }
+}
+
+
 void K3bIsoImager::calculateSize()
 {
-  // determine iso-size
-  cleanup();
-  init();
+  jobStarted();
+  startSizeCalculation();
+}
 
-  m_process = new K3bProcess();
-  m_process->setRunPrivileged(true);
 
-  d->mkisofsBin = k3bcore->externalBinManager()->binObject( "mkisofs" );
+void K3bIsoImager::startSizeCalculation()
+{
+  d->mkisofsBin = initMkisofs();
   if( !d->mkisofsBin ) {
-    kdDebug() << "(K3bIsoImager) could not find mkisofs executable" << endl;
-    emit infoMessage( i18n("Mkisofs executable not found."), K3bJob::ERROR );
-    cleanup();
-    emit sizeCalculated( ERROR, 0 );
+    jobFinished( false );
     return;
   }
+    
+  initVariables();
 
-  initMkisofs( d->mkisofsBin );
-
-//   emit debuggingOutput( "Used versions", "mkisofs: " + d->mkisofsBin->version );
-
-//   if( !d->mkisofsBin->copyright.isEmpty() )
-//     emit infoMessage( i18n("Using %1 %2 - Copyright (C) %3")
-// 		      .arg("mkisofs").arg(d->mkisofsBin->version).arg(d->mkisofsBin->copyright), INFO );
-
+  delete m_process;    
+  m_process = new K3bProcess();
+  m_process->setRunPrivileged(true);
+  m_process->setSplitStdout(true);
+    
+  emit debuggingOutput( "Used versions", "mkisofs: " + d->mkisofsBin->version );
+    
   *m_process << d->mkisofsBin;
 
-  // prepare the filenames as written to the image
-  m_doc->prepareFilenames();
-
   if( !prepareMkisofsFiles() || 
       !addMkisofsParameters(true) ) {
     cleanup();
-    emit sizeCalculated( ERROR, 0 );
+    jobFinished( false );
     return;
   }
 
@@ -261,7 +325,7 @@ void K3bIsoImager::calculateSize()
     s += *it + " ";
   }
   kdDebug() << s << endl << flush;
-
+  emit debuggingOutput("mkisofs calculate size command:", s);
 
   // since output changed during mkisofs version changes we grab both
   // stdout and stderr
@@ -274,13 +338,15 @@ void K3bIsoImager::calculateSize()
   // everything is written to stderr
   // last line is: "Total extents scheduled to be written = XXXXX"
 
+  // TODO: use K3bProcess::OutputCollector instead iof our own two slots.
+
   connect( m_process, SIGNAL(receivedStderr(KProcess*, char*, int)),
 	   this, SLOT(slotCollectMkisofsPrintSizeStderr(KProcess*, char*, int)) );
-  connect( m_process, SIGNAL(receivedStdout(KProcess*, char*, int)),
-	   this, SLOT(slotCollectMkisofsPrintSizeStdout(KProcess*, char*, int)) );
+  connect( m_process, SIGNAL(stdoutLine(const QString&)),
+	   this, SLOT(slotCollectMkisofsPrintSizeStdout(const QString&)) );
   connect( m_process, SIGNAL(processExited(KProcess*)),
 	   this, SLOT(slotMkisofsPrintSizeFinished()) );
-
+  
   // we also want error messages
   connect( m_process, SIGNAL(stderrLine( const QString& )),
 	   this, SLOT(slotReceivedStderr( const QString& )) );
@@ -293,7 +359,7 @@ void K3bIsoImager::calculateSize()
     emit infoMessage( i18n("Could not start %1.").arg("mkisofs"), K3bJob::ERROR );
     cleanup();
 
-    emit sizeCalculated( ERROR, 0 );
+    jobFinished( false );
     return;
   }
 }
@@ -306,22 +372,29 @@ void K3bIsoImager::slotCollectMkisofsPrintSizeStderr(KProcess*, char* data , int
 }
 
 
-void K3bIsoImager::slotCollectMkisofsPrintSizeStdout(KProcess*, char* data, int len )
+void K3bIsoImager::slotCollectMkisofsPrintSizeStdout( const QString& line )
 {
-  emit debuggingOutput( "mkisofs", QString::fromLocal8Bit( data, len ) );
-  m_collectedMkisofsPrintSizeStdout.append( QString::fromLocal8Bit( data, len ) );
+  // newer versions of mkisofs outut additional lines of junk before the size :(
+  // so we only use the last line
+  emit debuggingOutput( "mkisofs", line );
+  m_collectedMkisofsPrintSizeStdout = line;
 }
 
 
 void K3bIsoImager::slotMkisofsPrintSizeFinished()
 {
-  bool success = true;
+  if( m_canceled ) {
+    emit canceled();
+    jobFinished( false );
+    return;
+  }
 
-  kdDebug() << "(K3bIsoImager) iso size: " << m_collectedMkisofsPrintSizeStdout << endl;
+  bool success = true;
 
   // if m_collectedMkisofsPrintSizeStdout is not empty we have a recent version of
   // mkisofs and parsing is very easy (s.o.)
   if( !m_collectedMkisofsPrintSizeStdout.isEmpty() ) {
+    kdDebug() << "(K3bIsoImager) iso size: " << m_collectedMkisofsPrintSizeStdout << endl;
     m_mkisofsPrintSizeResult = m_collectedMkisofsPrintSizeStdout.toInt( &success );
   }
   else {
@@ -335,25 +408,32 @@ void K3bIsoImager::slotMkisofsPrintSizeFinished()
       m_mkisofsPrintSizeResult = m_collectedMkisofsPrintSizeStderr.mid( pos+33 ).toInt( &success );
   }
 
+  emit debuggingOutput( "K3bIsoImager", 
+			QString("mkisofs print size result: %1 (%2 bytes)")
+			.arg(m_mkisofsPrintSizeResult)
+			.arg(Q_UINT64(m_mkisofsPrintSizeResult)*2048ULL) );
+
   cleanup();
 
+
   if( success ) {
-    emit sizeCalculated( INFO, m_mkisofsPrintSizeResult );
+    jobFinished( true );
   }
   else {
     m_mkisofsPrintSizeResult = 0;
     kdDebug() << "(K3bIsoImager) Parsing mkisofs -print-size failed: " << m_collectedMkisofsPrintSizeStdout << endl;
     emit infoMessage( i18n("Could not determine size of resulting image file."), ERROR );
-    emit sizeCalculated( ERROR, 0 );
+    jobFinished( false );
   }
 }
 
 
-void K3bIsoImager::init()
+void K3bIsoImager::initVariables()
 {
   m_containsFilesWithMultibleBackslashes = false;
   m_processExited = false;
   m_canceled = false;
+  d->knownError = false;
 
   // determine symlink handling
   // follow links superseeds discard all links which superseeds discard broken links
@@ -378,28 +458,20 @@ void K3bIsoImager::init()
 
 void K3bIsoImager::start()
 {
-  emit started();
+  jobStarted();
 
   cleanup();
-  init();
 
-  m_process = new K3bProcess();
-  m_process->setRunPrivileged(true);
-
-  d->mkisofsBin = k3bcore->externalBinManager()->binObject( "mkisofs" );
+  d->mkisofsBin = initMkisofs();
   if( !d->mkisofsBin ) {
-    kdDebug() << "(K3bIsoImager) could not find mkisofs executable" << endl;
-    emit infoMessage( i18n("Could not find %1 executable.").arg("mkisofs"), K3bJob::ERROR );
-    cleanup();
-    emit finished( false );
+    jobFinished( false );
     return;
   }
 
-  if( !d->mkisofsBin->copyright.isEmpty() )
-    emit infoMessage( i18n("Using %1 %2 - Copyright (C) %3")
-		      .arg("mkisofs").arg(d->mkisofsBin->version).arg(d->mkisofsBin->copyright), INFO );
+  initVariables();
 
-  initMkisofs( d->mkisofsBin );
+  m_process = new K3bProcess();
+  m_process->setRunPrivileged(true);
 
   *m_process << d->mkisofsBin;
 
@@ -409,7 +481,7 @@ void K3bIsoImager::start()
   if( !prepareMkisofsFiles() ||
       !addMkisofsParameters() ) {
     cleanup();
-    emit finished( false );
+    jobFinished( false );
     return;
   }
 
@@ -419,20 +491,33 @@ void K3bIsoImager::start()
   connect( m_process, SIGNAL(stderrLine( const QString& )),
 	   this, SLOT(slotReceivedStderr( const QString& )) );
 
-  if( m_fdToWriteTo != -1 )
-    m_process->writeToFd( m_fdToWriteTo );
-  else {
+  //
+  // Check the image file
+  if( m_fdToWriteTo == -1 ) {
     d->imageFile.setName( d->imagePath );
-    if( d->imageFile.open( IO_WriteOnly ) )
-      m_process->writeToFd( d->imageFile.handle() );
-    else {
+    if( !d->imageFile.open( IO_WriteOnly ) ) {
       emit infoMessage( i18n("Could not open %1 for writing").arg(d->imagePath), ERROR );
       cleanup();
-      emit finished(false);
+      jobFinished(false);
       return;
     }
   }
 
+  //
+  // Open the active pipe which does the streaming
+  delete d->pipe;
+  if( m_doc->verifyData() )
+    d->pipe = new K3bChecksumPipe();
+  else
+    d->pipe = new K3bActivePipe();
+
+  if( m_fdToWriteTo == -1 )
+    d->pipe->writeToIODevice( &d->imageFile );
+  else
+    d->pipe->writeToFd( m_fdToWriteTo );
+  d->pipe->open();
+  m_process->writeToFd( d->pipe->in() );
+
 
   kdDebug() << "***** mkisofs parameters:\n";
   const QValueList<QCString>& args = m_process->args();
@@ -442,13 +527,13 @@ void K3bIsoImager::start()
   }
   kdDebug() << s << endl << flush;
   emit debuggingOutput("mkisofs command:", s);
-
+  
   if( !m_process->start( KProcess::NotifyOnExit, KProcess::AllOutput) ) {
     // something went wrong when starting the program
     // it "should" be the executable
     kdDebug() << "(K3bIsoImager) could not start mkisofs" << endl;
     emit infoMessage( i18n("Could not start %1.").arg("mkisofs"), K3bJob::ERROR );
-    emit finished( false );
+    jobFinished( false );
     cleanup();
   }
 }
@@ -458,15 +543,12 @@ void K3bIsoImager::cancel()
 {
   m_canceled = true;
 
-  if( m_process )
-    if( !m_processExited ) {
-      disconnect(m_process);
-      m_process->kill();
-    }
-
-  if( !m_processExited ) {
+  if( m_process && !m_processExited ) {
+    m_process->kill();
+  }
+  else if( active() ) {
     emit canceled();
-    emit finished(false);
+    jobFinished(false);
   }
 }
 
@@ -478,6 +560,17 @@ void K3bIsoImager::setMultiSessionInfo( const QString& info, K3bDevice::Device*
 }
 
 
+// iso9660 + RR use some latin1 variant. So we need to cut the desc fields
+// counting 8bit chars. The GUI should take care of restricting the length
+// and the charset
+static void truncateTheHardWay( QString& s, int max )
+{
+  QCString cs = s.utf8();
+  cs.truncate(max);
+  s = QString::fromUtf8( cs );
+}
+
+
 bool K3bIsoImager::addMkisofsParameters( bool printSize )
 {
   // add multisession info
@@ -496,7 +589,7 @@ bool K3bIsoImager::addMkisofsParameters( bool printSize )
 
   if( !m_doc->isoOptions().volumeID().isEmpty() ) {
     QString s = m_doc->isoOptions().volumeID();
-    s.truncate(32);  // ensure max length
+    truncateTheHardWay(s, 32);  // ensure max length
     *m_process << "-volid" << s;
   }
   else {
@@ -505,24 +598,36 @@ bool K3bIsoImager::addMkisofsParameters( bool printSize )
   }
 
   QString s = m_doc->isoOptions().volumeSetId();
-  s.truncate(128);  // ensure max length
+  truncateTheHardWay(s, 128);  // ensure max length
   *m_process << "-volset" << s;
-
+  
   s = m_doc->isoOptions().applicationID();
-  s.truncate(128);  // ensure max length
+  truncateTheHardWay(s, 128);  // ensure max length
   *m_process << "-appid" << s;
-
+  
   s = m_doc->isoOptions().publisher();
-  s.truncate(128);  // ensure max length
+  truncateTheHardWay(s, 128);  // ensure max length
   *m_process << "-publisher" << s;
-
+  
   s = m_doc->isoOptions().preparer();
-  s.truncate(128);  // ensure max length
+  truncateTheHardWay(s, 128);  // ensure max length
   *m_process << "-preparer" << s;
-
+  
   s = m_doc->isoOptions().systemId();
-  s.truncate(32);  // ensure max length
+  truncateTheHardWay(s, 32);  // ensure max length
   *m_process << "-sysid" << s;
+  
+  s = m_doc->isoOptions().abstractFile();
+  truncateTheHardWay(s, 37);  // ensure max length
+  *m_process << "-abstract" << s;
+
+  s = m_doc->isoOptions().copyrightFile();
+  truncateTheHardWay(s, 37);  // ensure max length
+  *m_process << "-copyright" << s;
+
+  s = m_doc->isoOptions().bibliographFile();
+  truncateTheHardWay(s, 37);  // ensure max length
+  *m_process << "-biblio" << s;
 
   int volsetSize = m_doc->isoOptions().volumeSetSize();
   int volsetSeqNo = m_doc->isoOptions().volumeSetNumber();
@@ -555,6 +660,9 @@ bool K3bIsoImager::addMkisofsParameters( bool printSize )
       *m_process << "-hide-joliet-list" << m_jolietHideFile->name();
   }
 
+  if( m_doc->isoOptions().doNotCacheInodes() )
+    *m_process << "-no-cache-inodes";
+
   //
   // Check if we have files > 2 GB and enable udf in that case.
   //
@@ -690,7 +798,7 @@ int K3bIsoImager::writePathSpec()
 
 int K3bIsoImager::writePathSpecForDir( K3bDirItem* dirItem, QTextStream& stream )
 {
-  if( dirItem->depth() > 7 ) {
+  if( !m_noDeepDirectoryRelocation && dirItem->depth() > 7 ) {
     kdDebug() << "(K3bIsoImager) found directory depth > 7. Enabling no deep directory relocation." << endl;
     m_noDeepDirectoryRelocation = true;
   }
@@ -716,7 +824,7 @@ int K3bIsoImager::writePathSpecForDir( K3bDirItem* dirItem, QTextStream& stream
 	  writeItem = false;
 	}
 	else if( f.isDir() ) {
-	  emit infoMessage( i18n("Ignoring link %1 to folder %2. K3b is not able to follow links to folders.")
+	  emit infoMessage( i18n("Ignoring link %1 to folder %2. K3b is unable to follow links to folders.")
 			    .arg(item->k3bName())
 			    .arg(f.filePath()), WARNING );
 	  writeItem = false;
@@ -747,7 +855,7 @@ int K3bIsoImager::writePathSpecForDir( K3bDirItem* dirItem, QTextStream& stream
       if( item->isDir() ) {
 	stream << escapeGraftPoint( item->writtenPath() )
 	       << "="
-	       << dummyDir( static_cast<K3bDirItem*>(item) ) << endl;
+	       << escapeGraftPoint( dummyDir( static_cast<K3bDirItem*>(item) ) ) << "\n";
 	
 	int x = writePathSpecForDir( dynamic_cast<K3bDirItem*>(item), stream );
 	if( x >= 0 )
@@ -785,12 +893,12 @@ void K3bIsoImager::writePathSpecForFile( K3bFileItem* item, QTextStream& stream
     static_cast<K3bBootItem*>(item)->setTempPath( tempPath );
     
     m_tempFiles.append(tempPath);
-    stream << escapeGraftPoint( tempPath ) << endl;
+    stream << escapeGraftPoint( tempPath ) << "\n";
   }
   else if( item->isSymLink() && d->usedLinkHandling == Private::FOLLOW )
-    stream << escapeGraftPoint( K3b::resolveLink( item->localPath() ) ) << endl;
+    stream << escapeGraftPoint( K3b::resolveLink( item->localPath() ) ) << "\n";
   else
-    stream << escapeGraftPoint( item->localPath() ) << endl;
+    stream << escapeGraftPoint( item->localPath() ) << "\n";
 }
 
 
@@ -853,7 +961,7 @@ bool K3bIsoImager::writeSortWeightFile()
   if( QTextStream* t = m_sortWeightFile->textStream() ) {
     //
     // We need to write the local path in combination with the sort weight
-    // mkisofs will take care of multible entries for one local file and always
+    // mkisofs will take care of multiple entries for one local file and always
     // use the highest weight
     //
     K3bDataItem* item = m_doc->root();
@@ -907,7 +1015,7 @@ QString K3bIsoImager::escapeGraftPoint( const QString& str )
   // time critical. :)
   //
 
-  int pos = 0;
+  unsigned int pos = 0;
   while( pos < enc.length() ) {
     // escape every equal sign with one backslash
     if( enc[pos] == '=' ) {
@@ -915,7 +1023,7 @@ QString K3bIsoImager::escapeGraftPoint( const QString& str )
       pos += 2;
     }
     else if( enc[pos] == '\\' ) {
-      // escape every occurence of two backslashes with two backslashes
+      // escape every occurrence of two backslashes with two backslashes
       if( pos+1 < enc.length() && enc[pos+1] == '\\' ) {
 	enc.insert( pos, "\\\\" );
 	pos += 4;
@@ -1004,7 +1112,7 @@ QString K3bIsoImager::dummyDir( K3bDirItem* dir )
   k3b_struct_stat statBuf;
   if( !dir->localPath().isEmpty() ) {
     // permissions
-    if( k3b_stat( QFile::encodeName( dir->localPath() ), &statBuf ) == 0 ) {
+    if( k3b_stat( QFile::encodeName(dir->localPath()), &statBuf ) == 0 ) {
       name += "_";
       name += QString::number( statBuf.st_uid );
       name += "_";
@@ -1020,6 +1128,9 @@ QString K3bIsoImager::dummyDir( K3bDirItem* dir )
 
 
   if( !_appDir.cd( name ) ) {
+
+    kdDebug() << "(K3bIsoImager) creating dummy dir: " << _appDir.absPath() << "/" << name << endl;
+
     _appDir.mkdir( name );
     _appDir.cd( name );
 
@@ -1049,4 +1160,19 @@ void K3bIsoImager::clearDummyDirs()
   }
 }
 
+
+QCString K3bIsoImager::checksum() const
+{
+  if( K3bChecksumPipe* p = dynamic_cast<K3bChecksumPipe*>( d->pipe ) )
+    return p->checksum();
+  else
+    return QCString();
+}
+
+
+bool K3bIsoImager::hasBeenCanceled() const
+{
+  return m_canceled;
+}
+
 #include "k3bisoimager.moc"
