@@ -1,10 +1,10 @@
 /*
  *
- * $Id: k3bcdrecordwriter.cpp 526655 2006-04-05 10:04:16Z trueg $
+ * $Id: k3bcdrecordwriter.cpp 630823 2007-02-06 14:07:10Z trueg $
  * Copyright (C) 2003 Sebastian Trueg <trueg@k3b.org>
  *
  * This file is part of the K3b project.
- * Copyright (C) 1998-2004 Sebastian Trueg <trueg@k3b.org>
+ * Copyright (C) 1998-2007 Sebastian Trueg <trueg@k3b.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -27,6 +27,7 @@
 #include <k3bglobals.h>
 #include <k3bthroughputestimator.h>
 #include <k3bglobalsettings.h>
+#include <k3btempfile.h>
 
 #include <qstring.h>
 #include <qstringlist.h>
@@ -37,11 +38,7 @@
 
 #include <klocale.h>
 #include <kdebug.h>
-#include <kmessagebox.h>
-#include <ktempfile.h>
-
-#include <errno.h>
-#include <string.h>
+#include <kglobal.h>
 
 
 
@@ -214,7 +211,7 @@ void K3bCdrecordWriter::prepareProcess()
       d->usingBurnfree = true;
 
       // with cdrecord 1.11a02 burnproof was renamed to burnfree
-      if( m_cdrecordBinObject->version < K3bVersion( "1.11a02" ) )
+      if( m_cdrecordBinObject->hasFeature( "burnproof" ) )
 	*m_process << "driveropts=burnproof";
       else
 	*m_process << "driveropts=burnfree";
@@ -222,6 +219,11 @@ void K3bCdrecordWriter::prepareProcess()
     else
       emit infoMessage( i18n("Writer does not support buffer underrun free recording (Burnfree)"), WARNING );
   }
+
+  if( k3bcore->globalSettings()->force() ) {
+    *m_process << "-force";
+    emit infoMessage( i18n("'Force unsafe operations' enabled."), WARNING );
+  }
   
   if( m_cue ) {
     m_process->setWorkingDirectory(QUrl(m_cueFile).dirPath());
@@ -233,7 +235,7 @@ void K3bCdrecordWriter::prepareProcess()
   
   if( m_rawCdText.size() > 0 ) {
     delete d->cdTextFile;
-    d->cdTextFile = new KTempFile( QString::null, ".dat" );
+    d->cdTextFile = new K3bTempFile( QString::null, ".dat" );
     d->cdTextFile->setAutoDelete(true);
     d->cdTextFile->file()->writeBlock( m_rawCdText );
     d->cdTextFile->close();
@@ -283,7 +285,7 @@ void K3bCdrecordWriter::clearArguments()
 
 void K3bCdrecordWriter::start()
 {
-  emit started();
+  jobStarted();
 
   d->canceled = false;
   d->speedEst->reset();
@@ -292,14 +294,17 @@ void K3bCdrecordWriter::start()
 
   if( !m_cdrecordBinObject ) {
     emit infoMessage( i18n("Could not find %1 executable.").arg("cdrecord"), ERROR );
-    emit finished(false);
+    jobFinished(false);
     return;
   }
 
   emit debuggingOutput( "Used versions", "cdrecord: " + m_cdrecordBinObject->version );
 
   if( !m_cdrecordBinObject->copyright.isEmpty() )
-    emit infoMessage( i18n("Using %1 %2 - Copyright (C) %3").arg(m_cdrecordBinObject->name()).arg(m_cdrecordBinObject->version).arg(m_cdrecordBinObject->copyright), INFO );
+    emit infoMessage( i18n("Using %1 %2 - Copyright (C) %3")
+		      .arg(m_cdrecordBinObject->hasFeature( "wodim" ) ? "Wodim" : "Cdrecord" )
+		      .arg(m_cdrecordBinObject->version)
+		      .arg(m_cdrecordBinObject->copyright), INFO );
 
 
   kdDebug() << "***** " << m_cdrecordBinObject->name() << " parameters:\n";
@@ -320,12 +325,19 @@ void K3bCdrecordWriter::start()
 
   emit newSubTask( i18n("Preparing write process...") );
 
+  // FIXME: check the return value
+  if( K3b::isMounted( burnDevice() ) ) {
+    emit infoMessage( i18n("Unmounting medium"), INFO );
+    K3b::unmount( burnDevice() );
+  }
+  k3bcore->blockDevice( burnDevice() );
+
   if( !m_process->start( KProcess::NotifyOnExit, KProcess::All ) ) {
     // something went wrong when starting the program
     // it "should" be the executable
     kdDebug() << "(K3bCdrecordWriter) could not start " << m_cdrecordBinObject->name() << endl;
     emit infoMessage( i18n("Could not start %1.").arg(m_cdrecordBinObject->name()), K3bJob::ERROR );
-    emit finished(false);
+    jobFinished(false);
   }
   else {
     if( simulate() ) {
@@ -474,10 +486,11 @@ void K3bCdrecordWriter::slotStdLine( const QString& line )
       m_cdrecordError = OPC_FAILED;
     }
     else if( errStr.startsWith( "Drive needs to reload the media" ) ) {
-      emit infoMessage( i18n("Reloading of media required"), K3bJob::INFO );
+      emit infoMessage( i18n("Reloading of medium required"), K3bJob::INFO );
     }
     else if( errStr.startsWith( "The current problem looks like a buffer underrun" ) ) {
-      m_cdrecordError = BUFFER_UNDERRUN;
+      if( m_cdrecordError == UNKNOWN ) // it is almost never a buffer underrun these days.
+	m_cdrecordError = BUFFER_UNDERRUN;
     }
     else if( errStr.startsWith("WARNING: Data may not fit") ) {
       bool overburn = k3bcore->globalSettings()->overburn();
@@ -502,8 +515,11 @@ void K3bCdrecordWriter::slotStdLine( const QString& line )
     else if( errStr.startsWith("Cannot send CUE sheet") ) {
       m_cdrecordError = CANNOT_SEND_CUE_SHEET;
     }
-    else if( errStr.startsWith( "Trying to use ultra high speed medium" ) ||
-	     errStr.startsWith( "Trying to use high speed medium" ) ) {
+    else if( errStr.startsWith( "Trying to use ultra high speed" ) ||
+	     errStr.startsWith( "Trying to use high speed" ) ||
+	     errStr.startsWith( "Probably trying to use ultra high speed" ) ||
+	     errStr.startsWith( "You did use a high speed medium on an improper writer" ) ||
+	     errStr.startsWith( "You did use a ultra high speed medium on an improper writer" ) ) {
       m_cdrecordError = HIGH_SPEED_MEDIUM;
     }
     else if( errStr.startsWith( "You may have used an ultra low speed medium" ) ) {
@@ -520,9 +536,12 @@ void K3bCdrecordWriter::slotStdLine( const QString& line )
       if( m_cdrecordError == UNKNOWN )
 	m_cdrecordError = CANNOT_FIXATE_DISK;
     }
-    else if( errStr == "A write error occured." ) {
+    else if( errStr == "A write error occurred." ) {
       m_cdrecordError = WRITE_ERROR;
     }
+    else if( errStr.startsWith( "Try again with cdrecord blank=all." ) ) {
+      m_cdrecordError = BLANK_FAILED;
+    }
   }
 
   //
@@ -551,7 +570,7 @@ void K3bCdrecordWriter::slotStdLine( const QString& line )
 	kdError() << "(K3bCdrecordWriter) Did not parse all tracks sizes!" << endl;
     }
     else
-      emit infoMessage( i18n("Starting writing"), INFO );
+      emit infoMessage( i18n("Starting disc write"), INFO );
 
     m_currentTrack++;
 
@@ -594,8 +613,8 @@ void K3bCdrecordWriter::slotStdLine( const QString& line )
   else if( line.startsWith( "Re-load disk and hit" ) ) {
     // this happens on some notebooks where cdrecord is not able to close the
     // tray itself, so we need to ask the user to do so
-    KMessageBox::information( 0, i18n("Please reload the medium and press 'ok'"),
-			      i18n("Unable to close the tray") );
+    blockingInformation( i18n("Please reload the medium and press 'ok'"),
+			 i18n("Unable to close the tray") );
 
     // now send a <CR> to cdrecord
     // hopefully this will do it since I have no possibility to test it!
@@ -616,7 +635,7 @@ void K3bCdrecordWriter::slotStdLine( const QString& line )
   else if( line.contains("Medium Error") ) {
     m_cdrecordError = MEDIUM_ERROR;
   }
-  else if( line.startsWith( "Error trying to open" ) && line.contains( "Device or resource busy" ) ) {
+  else if( line.startsWith( "Error trying to open" ) && line.contains( "(Device or resource busy)" ) ) {
     m_cdrecordError = DEVICE_BUSY;
   }
   else {
@@ -632,6 +651,8 @@ void K3bCdrecordWriter::slotProcessExited( KProcess* p )
   delete d->cdTextFile;
   d->cdTextFile = 0;
 
+  k3bcore->unblockDevice( burnDevice() );
+
   if( d->canceled ) {
     // this will unblock and eject the drive and emit the finished/canceled signals
     K3bAbstractWriter::cancel();
@@ -644,14 +665,14 @@ void K3bCdrecordWriter::slotProcessExited( KProcess* p )
     case 0:
       {
 	if( simulate() )
-	  emit infoMessage( i18n("Simulation successfully finished"), K3bJob::SUCCESS );
+	  emit infoMessage( i18n("Simulation successfully completed"), K3bJob::SUCCESS );
 	else
-	  emit infoMessage( i18n("Writing successfully finished"), K3bJob::SUCCESS );
+	  emit infoMessage( i18n("Writing successfully completed"), K3bJob::SUCCESS );
 	
 	int s = d->speedEst->average();
 	emit infoMessage( i18n("Average overall write speed: %1 KB/s (%2x)").arg(s).arg(KGlobal::locale()->formatNumber((double)s/150.0), 2), INFO );
 	
-	emit finished( true );
+	jobFinished( true );
       }
       break;
 
@@ -688,7 +709,8 @@ void K3bCdrecordWriter::slotProcessExited( KProcess* p )
 	break;
       case CANNOT_SEND_CUE_SHEET:
 	emit infoMessage( i18n("Unable to send CUE sheet."), ERROR );
-	emit infoMessage( i18n("This may be caused by wrong settings."), ERROR );
+	if( m_writingMode == K3b::DAO )
+	  emit infoMessage( i18n("Sometimes using TAO writing mode solves this issue."), ERROR );
 	break;
       case CANNOT_OPEN_NEW_SESSION:
 	emit infoMessage( i18n("Unable to open new session."), ERROR );
@@ -712,9 +734,11 @@ void K3bCdrecordWriter::slotProcessExited( KProcess* p )
 	break;
       case HIGH_SPEED_MEDIUM:
 	emit infoMessage( i18n("Found a high-speed medium not suitable for the writer being used."), ERROR );
+	emit infoMessage( i18n("Use the 'force unsafe operations' option to ignore this."), ERROR );
 	break;
       case LOW_SPEED_MEDIUM:
 	emit infoMessage( i18n("Found a low-speed medium not suitable for the writer being used."), ERROR );
+	emit infoMessage( i18n("Use the 'force unsafe operations' option to ignore this."), ERROR );
 	break;
       case MEDIUM_ERROR:
 	emit infoMessage( i18n("Most likely the burning failed due to low-quality media."), ERROR );
@@ -727,6 +751,10 @@ void K3bCdrecordWriter::slotProcessExited( KProcess* p )
 	if( m_writingMode == K3b::DAO )
 	  emit infoMessage( i18n("Sometimes using TAO writing mode solves this issue."), ERROR );
 	break;
+      case BLANK_FAILED:
+	emit infoMessage( i18n("Some drives do not support all erase types."), ERROR );
+	emit infoMessage( i18n("Try again using 'Complete' erasing."), ERROR );
+	break;
       case UNKNOWN:
 	if( p->exitStatus() == 12 && K3b::kernelVersion() >= K3bVersion( 2, 6, 8 ) && m_cdrecordBinObject->hasFeature( "suidroot" ) ) {
 	  emit infoMessage( i18n("Since kernel version 2.6.8 cdrecord cannot use SCSI transport when running suid root anymore."), ERROR );
@@ -736,7 +764,6 @@ void K3bCdrecordWriter::slotProcessExited( KProcess* p )
 	  emit infoMessage( i18n("%1 returned an unknown error (code %2).")
 			    .arg(m_cdrecordBinObject->name()).arg(p->exitStatus()), 
 			    K3bJob::ERROR );
-	  emit infoMessage( strerror(p->exitStatus()), K3bJob::ERROR );
 
 	  if( p->exitStatus() >= 254 && m_writingMode == K3b::DAO ) {
 	    emit infoMessage( i18n("Sometimes using TAO writing mode solves this issue."), ERROR );
@@ -750,13 +777,13 @@ void K3bCdrecordWriter::slotProcessExited( KProcess* p )
 	}
 	break;
       }
-      emit finished( false );
+      jobFinished( false );
     }
   }
   else {
     emit infoMessage( i18n("%1 did not exit cleanly.").arg(m_cdrecordBinObject->name()), 
 		      ERROR );
-    emit finished( false );
+    jobFinished( false );
   }
 }
 
