@@ -1,10 +1,10 @@
 /*
  *
- * $Id: k3bmixedjob.cpp 528555 2006-04-11 12:17:49Z trueg $
+ * $Id: k3bmixedjob.cpp 623768 2007-01-15 13:33:55Z trueg $
  * Copyright (C) 2003 Sebastian Trueg <trueg@k3b.org>
  *
  * This file is part of the K3b project.
- * Copyright (C) 1998-2004 Sebastian Trueg <trueg@k3b.org>
+ * Copyright (C) 1998-2007 Sebastian Trueg <trueg@k3b.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -41,6 +41,7 @@
 #include <k3btocfilewriter.h>
 #include <k3binffilewriter.h>
 #include <k3bglobalsettings.h>
+#include <k3baudiofile.h>
 
 #include <qfile.h>
 #include <qdatastream.h>
@@ -51,7 +52,31 @@
 #include <ktempfile.h>
 #include <kio/netaccess.h>
 #include <kio/global.h>
-#include <kmessagebox.h>
+#include <kstringhandler.h>
+
+
+static QString createNonExistingFilesString( const QValueList<K3bAudioFile*>& items, unsigned int max )
+{
+  QString s;
+  unsigned int cnt = 0;
+  for( QValueList<K3bAudioFile*>::const_iterator it = items.begin();
+       it != items.end(); ++it ) {
+
+    s += KStringHandler::csqueeze( (*it)->filename(), 60 );
+
+    ++cnt;
+    if( cnt >= max || it == items.end() )
+      break;
+
+    s += "<br>";
+  }
+
+  if( items.count() > max )
+    s += "...";
+
+  return s;
+}
+
 
 
 class K3bMixedJob::Private
@@ -61,6 +86,7 @@ public:
     : maxSpeedJob(0) {
   }
 
+
   int copies;
   int copiesDone;
 
@@ -77,7 +103,6 @@ K3bMixedJob::K3bMixedJob( K3bMixedDoc* doc, K3bJobHandler* hdl, QObject* parent
   d = new Private;
 
   m_isoImager = new K3bIsoImager( doc->dataDoc(), this, this );
-  connect( m_isoImager, SIGNAL(sizeCalculated(int, int)), this, SLOT(slotSizeCalculationFinished(int, int)) );
   connect( m_isoImager, SIGNAL(infoMessage(const QString&, int)), this, SIGNAL(infoMessage(const QString&, int)) );
   connect( m_isoImager, SIGNAL(percent(int)), this, SLOT(slotIsoImagerPercent(int)) );
   connect( m_isoImager, SIGNAL(finished(bool)), this, SLOT(slotIsoImagerFinished(bool)) );
@@ -111,7 +136,10 @@ K3bMixedJob::~K3bMixedJob()
 
 K3bDevice::Device* K3bMixedJob::writer() const
 {
-  return m_doc->burner();
+  if( m_doc->onlyCreateImages() )
+    return 0;
+  else
+    return m_doc->burner();
 }
 
 
@@ -123,7 +151,7 @@ K3bDoc* K3bMixedJob::doc() const
 
 void K3bMixedJob::start()
 {
-  emit started();
+  jobStarted();
 
   m_canceled = false;
   m_errorOccuredAndAlreadyReported = false;
@@ -137,6 +165,52 @@ void K3bMixedJob::start()
 
   prepareProgressInformation();
 
+  //
+  // Check if all files exist
+  //
+  QValueList<K3bAudioFile*> nonExistingFiles;
+  K3bAudioTrack* track = m_doc->audioDoc()->firstTrack();
+  while( track ) {
+    K3bAudioDataSource* source = track->firstSource();
+    while( source ) {
+      if( K3bAudioFile* file = dynamic_cast<K3bAudioFile*>( source ) ) {
+	if( !QFile::exists( file->filename() ) )
+	  nonExistingFiles.append( file );
+      }
+      source = source->next();
+    }
+    track = track->next();
+  }
+  if( !nonExistingFiles.isEmpty() ) {
+    if( questionYesNo( "<p>" + i18n("The following files could not be found. Do you want to remove them from the "
+				    "project and continue without adding them to the image?") + 
+		       "<p>" + createNonExistingFilesString( nonExistingFiles, 10 ),
+		       i18n("Warning"),
+		       i18n("Remove missing files and continue"),
+		       i18n("Cancel and go back") ) ) {
+      for( QValueList<K3bAudioFile*>::const_iterator it = nonExistingFiles.begin();
+	   it != nonExistingFiles.end(); ++it ) {
+	delete *it;
+      }
+    }
+    else {
+      m_canceled = true;
+      emit canceled();
+      jobFinished(false);
+      return;
+    }
+  }
+
+  //
+  // Make sure the project is not empty
+  //
+  if( m_doc->audioDoc()->numOfTracks() == 0 ) {
+    emit infoMessage( i18n("Please add files to your project first."), ERROR );
+    jobFinished(false);
+    return;
+  }
+
+
   // set some flags that are needed
   m_doc->audioDoc()->setOnTheFly( m_doc->onTheFly() );  // for the toc writer
   m_doc->audioDoc()->setHideFirstTrack( false );   // unsupported
@@ -144,38 +218,16 @@ void K3bMixedJob::start()
 
   emit newTask( i18n("Preparing data") );
 
-  m_doc->dataDoc()->prepareFilenames();
-  if( m_doc->dataDoc()->needToCutFilenames() ) {
-    if( !questionYesNo( i18n("Some filenames need to be shortened due to the %1 char restriction "
-			     "of the Joliet extensions. Continue anyway?")
-			.arg( m_doc->dataDoc()->isoOptions().jolietLong() ? 103 : 64 ) ) ) {
-      emit canceled();
-      emit finished( false );
-      return;
-    }
-  }
-
   determineWritingMode();
 
-  // depending on the mixed type and if it's onthefly or not we
-  // decide what to do first
-
   //
-  // in case we write two sessions we need to check if the resulting data will really fit on the Cd
+  // First we make sure the data portion is valid
   //
-  if( m_doc->mixedType() == K3bMixedDoc::DATA_SECOND_SESSION )
-    determinePreliminaryDataImageSize();
-  else
-    startFirstCopy();
-}
 
-
-
-void K3bMixedJob::determinePreliminaryDataImageSize()
-{
   // we do not have msinfo yet
+  m_currentAction = INITIALIZING_IMAGER;
   m_isoImager->setMultiSessionInfo( QString::null );
-  m_isoImager->calculateSize();
+  m_isoImager->init();
 }
 
 
@@ -207,6 +259,7 @@ void K3bMixedJob::startFirstCopy()
       d->maxSpeedJob->start();
     }
     else if( m_doc->mixedType() != K3bMixedDoc::DATA_SECOND_SESSION ) {
+      m_currentAction = PREPARING_DATA;
       m_isoImager->calculateSize();
     }
     else {
@@ -250,6 +303,7 @@ void K3bMixedJob::slotMaxSpeedJobFinished( bool success )
     emit infoMessage( i18n("Unable to determine maximum speed for some reason. Ignoring."), WARNING );
 
   if( m_doc->mixedType() != K3bMixedDoc::DATA_SECOND_SESSION ) {
+    m_currentAction = PREPARING_DATA;
     m_isoImager->calculateSize();
   }
   else {
@@ -266,7 +320,7 @@ void K3bMixedJob::writeNextCopy()
     m_currentAction = WRITING_AUDIO_IMAGE;
     if( !prepareWriter() || !startWriting() ) {
       cleanupAfterError();
-      emit finished(false);
+      jobFinished(false);
     }
     else if( m_doc->onTheFly() )
       m_audioImager->start();
@@ -280,7 +334,7 @@ void K3bMixedJob::writeNextCopy()
 
     if( !prepareWriter() || !startWriting() ) {
       cleanupAfterError();
-      emit finished(false);
+      jobFinished(false);
     }
     else if( m_doc->onTheFly() ) {
       if( m_doc->mixedType() == K3bMixedDoc::DATA_LAST_TRACK )
@@ -307,7 +361,7 @@ void K3bMixedJob::cancel()
   emit infoMessage( i18n("Writing canceled."), K3bJob::ERROR );
   removeBufferFiles();
   emit canceled();
-  emit finished(false);
+  jobFinished(false);
 }
 
 
@@ -325,6 +379,7 @@ void K3bMixedJob::slotMsInfoFetched( bool success )
 					.arg(m_msInfoFetcher->nextSessionStart()+150) );
 
     if( m_doc->onTheFly() ) {
+      m_currentAction = PREPARING_DATA;
       m_isoImager->calculateSize();
     }
     else {
@@ -334,53 +389,7 @@ void K3bMixedJob::slotMsInfoFetched( bool success )
   else {
     // the MsInfoFetcher already emitted failure info
     cleanupAfterError();
-    emit finished(false);
-  }
-}
-
-
-void K3bMixedJob::slotSizeCalculationFinished( int status, int size )
-{
-  kdDebug() << "(K3bMixedJob) size calculated: " << size << endl;
-  emit debuggingOutput( "K3b", QString( "Size of filesystem calculated: %1" ).arg(size) );
-
-  if( status != ERROR ) {
-
-    if( m_currentAction == PREPARING_DATA ) {
-      // check the size
-      m_projectSize  = size + m_doc->audioDoc()->length();
-      if( m_doc->mixedType() == K3bMixedDoc::DATA_SECOND_SESSION )
-	m_projectSize += 11400; // the session gap
-      
-      startFirstCopy();
-    }
-
-    // 1. data in first track:
-    //    start isoimager and writer
-    //    when isoimager finishes start audiodecoder
-
-    // 2. data in last track
-    //    start audiodecoder and writer
-    //    when audiodecoder finishes start isoimager
-
-    // 3. data in second session
-    //    start audiodecoder and writer
-    //    start isoimager and writer
-
-    else if( m_doc->mixedType() == K3bMixedDoc::DATA_SECOND_SESSION ) {
-      if( !prepareWriter() || !startWriting() ) {
-	cleanupAfterError();
-	emit finished(false);
-      }
-      else
-	m_isoImager->start();
-    }
-    else
-      writeNextCopy();
-  }
-  else {
-    cleanupAfterError();
-    emit finished( false );
+    jobFinished(false);
   }
 }
 
@@ -390,35 +399,95 @@ void K3bMixedJob::slotIsoImagerFinished( bool success )
   if( m_canceled || m_errorOccuredAndAlreadyReported )
     return;
 
-  if( !success ) {
-    emit infoMessage( i18n("Error while creating ISO image."), ERROR );
-    cleanupAfterError();
-
-    emit finished( false );
-    return;
-  }
-
-  if( m_doc->onTheFly() ) {
-    if( m_doc->mixedType() == K3bMixedDoc::DATA_FIRST_TRACK ) {
-      m_currentAction = WRITING_AUDIO_IMAGE;
-      m_audioImager->start();
+  //
+  // Initializing imager before the first copy
+  //
+  if( m_currentAction == INITIALIZING_IMAGER ) {
+    if( success ) {
+      m_currentAction = PREPARING_DATA;
+      
+      // check the size
+      m_projectSize  = m_isoImager->size() + m_doc->audioDoc()->length();
+      if( m_doc->mixedType() == K3bMixedDoc::DATA_SECOND_SESSION )
+	m_projectSize += 11400; // the session gap
+      
+      startFirstCopy();
+    }
+    else {
+      cleanupAfterError();
+      jobFinished( false );
     }
   }
+
+  //
+  // Recalculated iso image size
+  //
+  else if( m_currentAction == PREPARING_DATA ) {
+    if( success ) {
+      // 1. data in first track:
+      //    start isoimager and writer
+      //    when isoimager finishes start audiodecoder
+      
+      // 2. data in last track
+      //    start audiodecoder and writer
+      //    when audiodecoder finishes start isoimager
+      
+      // 3. data in second session
+      //    start audiodecoder and writer
+      //    start isoimager and writer
+      
+      if( m_doc->mixedType() == K3bMixedDoc::DATA_SECOND_SESSION ) {
+	m_currentAction = WRITING_ISO_IMAGE;
+	if( !prepareWriter() || !startWriting() ) {
+	  cleanupAfterError();
+	  jobFinished(false);
+	}
+	else
+	  m_isoImager->start();
+      }
+      else
+	writeNextCopy();
+    }
+    else {
+      cleanupAfterError();
+      jobFinished( false );
+    }
+  }
+
+  //
+  // Image creation finished
+  //
   else {
-    emit infoMessage( i18n("ISO image successfully created."), SUCCESS );
+    if( !success ) {
+      emit infoMessage( i18n("Error while creating ISO image."), ERROR );
+      cleanupAfterError();
 
-    if( m_doc->mixedType() == K3bMixedDoc::DATA_SECOND_SESSION ) {
-      m_currentAction = WRITING_ISO_IMAGE;
+      jobFinished( false );
+      return;
+    }
 
-      if( !prepareWriter() || !startWriting() ) {
-	cleanupAfterError();
-	emit finished(false);
+    if( m_doc->onTheFly() ) {
+      if( m_doc->mixedType() == K3bMixedDoc::DATA_FIRST_TRACK ) {
+	m_currentAction = WRITING_AUDIO_IMAGE;
+	m_audioImager->start();
       }
     }
     else {
-      emit newTask( i18n("Creating audio image files") );
-      m_currentAction = CREATING_AUDIO_IMAGE;
-      m_audioImager->start();
+      emit infoMessage( i18n("ISO image successfully created."), SUCCESS );
+
+      if( m_doc->mixedType() == K3bMixedDoc::DATA_SECOND_SESSION ) {
+	m_currentAction = WRITING_ISO_IMAGE;
+
+	if( !prepareWriter() || !startWriting() ) {
+	  cleanupAfterError();
+	  jobFinished(false);
+	}
+      }
+      else {
+	emit newTask( i18n("Creating audio image files") );
+	m_currentAction = CREATING_AUDIO_IMAGE;
+	m_audioImager->start();
+      }
     }
   }
 }
@@ -431,7 +500,7 @@ void K3bMixedJob::slotWriterFinished( bool success )
 
   if( !success ) {
     cleanupAfterError();
-    emit finished(false);
+    jobFinished(false);
     return;
   }
 
@@ -440,7 +509,7 @@ void K3bMixedJob::slotWriterFinished( bool success )
   if( m_doc->mixedType() == K3bMixedDoc::DATA_SECOND_SESSION && m_currentAction == WRITING_AUDIO_IMAGE ) {
     // reload the media (as a subtask so the user does not see the "Flushing cache" or "Fixating" messages while
     // doing so
-    emit newSubTask( i18n("Reloading the media") );
+    emit newSubTask( i18n("Reloading the medium") );
     connect( K3bDevice::reload( m_doc->burner() ), SIGNAL(finished(bool)),
 	     this, SLOT(slotMediaReloadedForSecondSession(bool)) );
   }
@@ -454,7 +523,7 @@ void K3bMixedJob::slotWriterFinished( bool success )
       if( !m_doc->onTheFly() && m_doc->removeImages() )
 	removeBufferFiles();
       
-      emit finished(true);
+      jobFinished(true);
     }
   }
 }
@@ -463,8 +532,8 @@ void K3bMixedJob::slotWriterFinished( bool success )
 void K3bMixedJob::slotMediaReloadedForSecondSession( bool success )
 {
   if( !success )
-    KMessageBox::information( 0, i18n("Please reload the medium and press 'ok'"),
-			      i18n("Unable to close the tray") );
+    blockingInformation( i18n("Please reload the medium and press 'ok'"),
+			 i18n("Unable to close the tray") );
 
   // start the next session
   m_currentAction = WRITING_ISO_IMAGE;
@@ -472,7 +541,7 @@ void K3bMixedJob::slotMediaReloadedForSecondSession( bool success )
     // we only create the image once. This should not be a problem???
     if( !prepareWriter() || !startWriting() ) {
       cleanupAfterError();
-      emit finished(false);
+      jobFinished(false);
     }
     else if( m_doc->onTheFly() ) {
       m_isoImager->start();
@@ -480,12 +549,15 @@ void K3bMixedJob::slotMediaReloadedForSecondSession( bool success )
   }
   else if( m_doc->dummy() ) {
     // do not try to get ms info in simulation mode since the cd is empty!
-    if( m_doc->onTheFly() )
+    if( m_doc->onTheFly() ) {
+      m_currentAction = PREPARING_DATA;
       m_isoImager->calculateSize();
+    }
     else 
       createIsoImage();
   }
   else {
+    m_currentAction = FETCHING_MSINFO;
     m_msInfoFetcher->setDevice( m_doc->burner() );
     m_msInfoFetcher->start();
   }
@@ -500,7 +572,7 @@ void K3bMixedJob::slotAudioDecoderFinished( bool success )
   if( !success ) {
     emit infoMessage( i18n("Error while decoding audio tracks."), ERROR );
     cleanupAfterError();
-    emit finished(false);
+    jobFinished(false);
     return;
   }
 
@@ -524,7 +596,7 @@ void K3bMixedJob::slotAudioDecoderFinished( bool success )
 
       if( !prepareWriter() || !startWriting() ) {
 	cleanupAfterError();
-	emit finished(false);
+	jobFinished(false);
       }
     }
   }
@@ -739,8 +811,9 @@ void K3bMixedJob::addAudioTracks( K3bCdrecordWriter* writer )
 
   writer->addArgument( "-audio" );
 
-  // we always pad because although K3b makes sure all tracks' lenght are multible of 2352
+  // we always pad because although K3b makes sure all tracks' length are multiples of 2352
   // it seems that normalize sometimes corrupts these lengths
+  // FIXME: see K3bAudioJob for the whole less4secs and zeroPregap handling
   writer->addArgument( "-pad" );
 
   // Allow tracks shorter than 4 seconds
@@ -963,7 +1036,7 @@ bool K3bMixedJob::startWriting()
   }
 
   // in case we determined the max possible writing speed we have to reset the speed on the writer job
-  // here since an inserted media is neccessary
+  // here since an inserted media is necessary
   // the Max speed job will compare the max speed value with the supported values of the writer
   if( d->maxSpeed )
     m_writer->setBurnSpeed( d->maxSpeedJob->maxSpeed() );
@@ -1056,8 +1129,7 @@ void K3bMixedJob::determineWritingMode()
   
   if( k3bcore->externalBinManager()->binObject("cdrecord") ) {
     cdrecordOnTheFly =
-      k3bcore->externalBinManager()->binObject("cdrecord")->version
-      >= K3bVersion( 2, 1, -1, "a13" );
+      k3bcore->externalBinManager()->binObject("cdrecord")->hasFeature( "audio-stdin" );
     cdrecordCdText =
       k3bcore->externalBinManager()->binObject("cdrecord")->hasFeature( "cdtext" );
     cdrecordUsable =
@@ -1187,12 +1259,12 @@ void K3bMixedJob::slotNormalizeJobFinished( bool success )
 
     if( !prepareWriter() || !startWriting() ) {
       cleanupAfterError();
-      emit finished(false);
+      jobFinished(false);
     }
   }
   else {
     cleanupAfterError();
-    emit finished(false);
+    jobFinished(false);
   }
 }
 
