@@ -1,10 +1,10 @@
 /* 
  *
- * $Id: k3bprojectinterface.h 485537 2005-12-04 20:33:02Z trueg $
+ * $Id: k3bprojectinterface.h 619556 2007-01-03 17:38:12Z trueg $
  * Copyright (C) 2003 Sebastian Trueg <trueg@k3b.org>
  *
  * This file is part of the K3b project.
- * Copyright (C) 1998-2004 Sebastian Trueg <trueg@k3b.org>
+ * Copyright (C) 1998-2007 Sebastian Trueg <trueg@k3b.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,8 +18,8 @@
 #define _K3B_PROJECT_INTERFACE_H_
 
 #include <dcopobject.h>
-#include <kurl.h>
 #include <kio/global.h>
+#include <qstringlist.h>
 
 class K3bDoc;
 
@@ -39,8 +39,8 @@ class K3bProjectInterface : public DCOPObject
   static QCString newIfaceName();
 
  k_dcop:
-  virtual void addUrls( const KURL::List& urls );
-  virtual void addUrl( const KURL& url );
+  virtual void addUrls( const QStringList& urls );
+  virtual void addUrl( const QString& url );
 
   /**
    * Opens the burn dialog
@@ -49,8 +49,11 @@ class K3bProjectInterface : public DCOPObject
 
   /**
    * Starts the burning immedeately
+   * \return true if the burning could be started. Be aware that the return
+   *         value does not say anything about the success of the burning
+   *         process.
    */
-  virtual void directBurn();
+  virtual bool directBurn();
 
   virtual void setBurnDevice( const QString& blockdevicename );
 
