@@ -1,11 +1,11 @@
 /*
  *
- * $Id: k3bcdcopydialog.cpp 471081 2005-10-16 10:55:47Z trueg $
+ * $Id: k3bcdcopydialog.cpp 619556 2007-01-03 17:38:12Z trueg $
  * Copyright (C) 2003 Sebastian Trueg <trueg@k3b.org>
  *                    Klaus-Dieter Krannich <kd@k3b.org>
  *
  * This file is part of the K3b project.
- * Copyright (C) 1998-2004 Sebastian Trueg <trueg@k3b.org>
+ * Copyright (C) 1998-2007 Sebastian Trueg <trueg@k3b.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
 
 #include "k3bcdcopydialog.h"
 
-#include "k3bdevicecombobox.h"
+#include "k3bmediaselectioncombobox.h"
 #include "k3bcdcopyjob.h"
 #include "k3bclonejob.h"
 
@@ -32,6 +32,8 @@
 #include <k3bexternalbinmanager.h>
 #include <k3bthememanager.h>
 #include <k3bwritingmodewidget.h>
+#include <k3bapplication.h>
+#include <k3bmediacache.h>
 
 #include <kguiitem.h>
 #include <klocale.h>
@@ -74,13 +76,16 @@ K3bCdCopyDialog::K3bCdCopyDialog( QWidget *parent, const char *name, bool modal
   mainGrid->setSpacing( spacingHint() );
   mainGrid->setMargin( 0 );
 
-  QGroupBox* groupSource = new QGroupBox( 1, Qt::Vertical, i18n("CD Reader Device"), main );
+  QGroupBox* groupSource = new QGroupBox( 1, Qt::Vertical, i18n("Source Medium"), main );
   groupSource->setInsideSpacing( spacingHint() );
   groupSource->setInsideMargin( marginHint() );
-  m_comboSourceDevice = new K3bDeviceComboBox( groupSource );
-  m_comboSourceDevice->addDevices( k3bcore->deviceManager()->cdReader() );
+  m_comboSourceDevice = new K3bMediaSelectionComboBox( groupSource );
+  m_comboSourceDevice->setWantedMediumType( K3bDevice::MEDIA_WRITABLE_CD|K3bDevice::MEDIA_CD_ROM );
+  m_comboSourceDevice->setWantedMediumState( K3bDevice::STATE_COMPLETE|K3bDevice::STATE_INCOMPLETE );
 
-  m_writerSelectionWidget = new K3bWriterSelectionWidget( false, main );
+  m_writerSelectionWidget = new K3bWriterSelectionWidget( main );
+  m_writerSelectionWidget->setWantedMediumType( K3bDevice::MEDIA_WRITABLE_CD );
+  m_writerSelectionWidget->setWantedMediumState( K3bDevice::STATE_EMPTY );
   m_writerSelectionWidget->setSupportedWritingApps( K3b::CDRECORD );
 
   // tab widget --------------------
@@ -100,9 +105,6 @@ K3bCdCopyDialog::K3bCdCopyDialog( QWidget *parent, const char *name, bool modal
   m_comboCopyMode = new QComboBox( groupCopyMode );
   m_comboCopyMode->insertItem( i18n("Normal Copy") );
   m_comboCopyMode->insertItem( i18n("Clone Copy") );
-//   m_groupCopyMode->setExclusive( true );
-//   m_radioNormalCopy = new QRadioButton( i18n("Normal copy"), m_groupCopyMode );
-//   m_radioCloneCopy = new QRadioButton( i18n("Clone copy"), m_groupCopyMode );
 
   QGroupBox* groupWritingMode = new QGroupBox( 1, Qt::Vertical, i18n("Writing Mode"), optionTab );
   groupWritingMode->setInsideMargin( marginHint() );
@@ -116,11 +118,11 @@ K3bCdCopyDialog::K3bCdCopyDialog( QWidget *parent, const char *name, bool modal
   pixLabel->setScaledContents( false );
   m_spinCopies = new QSpinBox( 1, 999, 1, groupCopies );
 
-  QGroupBox* groupOptions = new QGroupBox( 5, Qt::Vertical, i18n("Options"), optionTab );
+  QGroupBox* groupOptions = new QGroupBox( 5, Qt::Vertical, i18n("Settings"), optionTab );
   groupOptions->setInsideSpacing( spacingHint() );
   groupOptions->setInsideMargin( marginHint() );
   m_checkSimulate = K3bStdGuiItems::simulateCheckbox( groupOptions );
-  m_checkOnTheFly = K3bStdGuiItems::onTheFlyCheckbox( groupOptions );
+  m_checkCacheImage = K3bStdGuiItems::createCacheImageCheckbox( groupOptions );
   m_checkOnlyCreateImage = K3bStdGuiItems::onlyCreateImagesCheckbox( groupOptions );
   m_checkDeleteImages = K3bStdGuiItems::removeImagesCheckbox( groupOptions );
 
@@ -157,35 +159,33 @@ K3bCdCopyDialog::K3bCdCopyDialog( QWidget *parent, const char *name, bool modal
   advancedTabGrid->setSpacing( spacingHint() );
   advancedTabGrid->setMargin( marginHint() );
 
-  QGroupBox* groupGeneral = new QGroupBox( 2, Qt::Vertical, i18n("General"), advancedTab ); 
-  groupGeneral->setInsideSpacing( spacingHint() );
-  groupGeneral->setInsideMargin( marginHint() );
-  QHBox* box = new QHBox( groupGeneral );
+  m_groupAdvancedDataOptions = new QGroupBox( 3, Qt::Vertical, i18n("Data"), advancedTab, "data_options" ); 
+  m_groupAdvancedDataOptions->setInsideSpacing( spacingHint() );
+  m_groupAdvancedDataOptions->setInsideMargin( marginHint() );
+  QHBox* box = new QHBox( m_groupAdvancedDataOptions );
   box->setSpacing( spacingHint() );
   box->setStretchFactor( new QLabel( i18n("Read retries:"), box ), 1 );
-  m_spinRetries = new QSpinBox( 1, 128, 1, box );
+  m_spinDataRetries = new QSpinBox( 1, 128, 1, box );
+  m_checkIgnoreDataReadErrors = K3bStdGuiItems::ignoreAudioReadErrorsCheckBox( m_groupAdvancedDataOptions );
+  m_checkNoCorrection = new QCheckBox( i18n("No error correction"), m_groupAdvancedDataOptions );
 
-  m_checkIgnoreReadErrors = new QCheckBox( i18n("Ignore read errors"), groupGeneral );
-
-  QGroupBox* groupData = new QGroupBox( 1, Qt::Vertical, i18n("Data"), advancedTab ); 
-  groupData->setInsideSpacing( spacingHint() );
-  groupData->setInsideMargin( marginHint() );
-  m_checkNoCorrection = new QCheckBox( i18n("No error correction"), groupData );
-
-  QGroupBox* groupAudio = new QGroupBox( 3, Qt::Vertical, i18n("Audio"), advancedTab ); 
-  groupAudio->setInsideSpacing( spacingHint() );
-  groupAudio->setInsideMargin( marginHint() );
-  box = new QHBox( groupAudio );
+  m_groupAdvancedAudioOptions = new QGroupBox( 5, Qt::Vertical, i18n("Audio"), advancedTab, "audio_options" ); 
+  m_groupAdvancedAudioOptions->setInsideSpacing( spacingHint() );
+  m_groupAdvancedAudioOptions->setInsideMargin( marginHint() );
+  box = new QHBox( m_groupAdvancedAudioOptions );
+  box->setSpacing( spacingHint() );
+  box->setStretchFactor( new QLabel( i18n("Read retries:"), box ), 1 );
+  m_spinAudioRetries = new QSpinBox( 1, 128, 1, box );
+  m_checkIgnoreAudioReadErrors = K3bStdGuiItems::ignoreAudioReadErrorsCheckBox( m_groupAdvancedAudioOptions );
+  box = new QHBox( m_groupAdvancedAudioOptions );
   box->setSpacing( spacingHint() );
   box->setStretchFactor(new QLabel( i18n("Paranoia mode:"), box ), 1 );
   m_comboParanoiaMode = K3bStdGuiItems::paranoiaModeComboBox( box );
-  m_checkReadCdText = new QCheckBox( i18n("Copy CD-Text"), groupAudio );
-  m_checkPrefereCdText = new QCheckBox( i18n("Prefer CD-Text"), groupAudio );
+  m_checkReadCdText = new QCheckBox( i18n("Copy CD-Text"), m_groupAdvancedAudioOptions );
+  m_checkPrefereCdText = new QCheckBox( i18n("Prefer CD-Text"), m_groupAdvancedAudioOptions );
 
-  advancedTabGrid->addWidget( groupGeneral, 0, 0 );
-  advancedTabGrid->addWidget( groupData, 1, 0 );
-  advancedTabGrid->addMultiCellWidget( groupAudio, 0, 1, 1, 1 );  
-  advancedTabGrid->setRowStretch( 1, 1 );
+  advancedTabGrid->addWidget( m_groupAdvancedDataOptions, 0, 1 );
+  advancedTabGrid->addWidget( m_groupAdvancedAudioOptions, 0, 0 );  
 
   tabWidget->addTab( advancedTab, i18n("&Advanced") );
 
@@ -196,18 +196,19 @@ K3bCdCopyDialog::K3bCdCopyDialog( QWidget *parent, const char *name, bool modal
 
 
   connect( m_comboSourceDevice, SIGNAL(selectionChanged(K3bDevice::Device*)), this, SLOT(slotToggleAll()) );
+  connect( m_comboSourceDevice, SIGNAL(selectionChanged(K3bDevice::Device*)), 
+	   this, SLOT(slotSourceMediumChanged(K3bDevice::Device*)) );
   connect( m_writerSelectionWidget, SIGNAL(writerChanged()), this, SLOT(slotToggleAll()) );
+  connect( m_writerSelectionWidget, SIGNAL(writerChanged(K3bDevice::Device*)), 
+	   m_writingModeWidget, SLOT(setDevice(K3bDevice::Device*)) );
   connect( m_writingModeWidget, SIGNAL(writingModeChanged(int)), this, SLOT(slotToggleAll()) );
-  connect( m_checkOnTheFly, SIGNAL(toggled(bool)), this, SLOT(slotToggleAll()) );
+  connect( m_checkCacheImage, SIGNAL(toggled(bool)), this, SLOT(slotToggleAll()) );
   connect( m_checkSimulate, SIGNAL(toggled(bool)), this, SLOT(slotToggleAll()) );
   connect( m_checkOnlyCreateImage, SIGNAL(toggled(bool)), this, SLOT(slotToggleAll()) );
   connect( m_comboCopyMode, SIGNAL(activated(int)), this, SLOT(slotToggleAll()) );
   connect( m_checkReadCdText, SIGNAL(toggled(bool)), this, SLOT(slotToggleAll()) );
 
-  connect( k3bcore->deviceManager(), SIGNAL(changed(K3bDevice::DeviceManager*)),
-	   this, SLOT(slotDeviceManagerChanged(K3bDevice::DeviceManager*)) );
-
-  QToolTip::add( m_checkIgnoreReadErrors, i18n("Skip unreadable sectors") );
+  QToolTip::add( m_checkIgnoreDataReadErrors, i18n("Skip unreadable data sectors") );
   QToolTip::add( m_checkNoCorrection, i18n("Disable the source drive's error correction") );
   QToolTip::add( m_checkPrefereCdText, i18n("Use CD-Text instead of cddb if available.") );
   QToolTip::add( m_checkReadCdText, i18n("Copy CD-Text from the source CD if available.") );
@@ -222,9 +223,8 @@ K3bCdCopyDialog::K3bCdCopyDialog( QWidget *parent, const char *name, bool modal
 					   "to stick to Cddb info.") );
   QWhatsThis::add( m_checkPrefereCdText, i18n("<p>If this option is checked and K3b finds CD-Text on the source media it will be "
 					      "copied to the resulting CD ignoring any potentially existing Cddb entries.") );
-  QWhatsThis::add( m_checkIgnoreReadErrors, i18n("<p>If this option is checked and K3b is not able to read a sector from the "
-						 "source CD/DVD it will be replaced with zeros on the resulting copy.") );
-
+  QWhatsThis::add( m_checkIgnoreDataReadErrors, i18n("<p>If this option is checked and K3b is not able to read a data sector from the "
+						     "source CD/DVD it will be replaced with zeros on the resulting copy.") );
 
   QWhatsThis::add( m_comboCopyMode, 
 		   "<p><b>" + i18n("Normal Copy") + "</b>"
@@ -247,8 +247,13 @@ K3bCdCopyDialog::~K3bCdCopyDialog()
 
 void K3bCdCopyDialog::init()
 {
-  if( !m_writerSelectionWidget->writerDevice() )
-    m_checkOnlyCreateImage->setChecked( true );
+  slotSourceMediumChanged( m_comboSourceDevice->selectedDevice() );
+}
+
+
+void K3bCdCopyDialog::setReadingDevice( K3bDevice::Device* dev )
+{
+  m_comboSourceDevice->setSelectedDevice( dev );
 }
 
 
@@ -260,6 +265,18 @@ K3bDevice::Device* K3bCdCopyDialog::readingDevice() const
 
 void K3bCdCopyDialog::slotStartClicked()
 {
+  //
+  // Let's check the available size
+  //
+  if( m_checkCacheImage->isChecked() || m_checkOnlyCreateImage->isChecked() ) {
+    if( neededSize()/1024 > m_tempDirSelectionWidget->freeTempSpace() ) {
+      if( KMessageBox::warningContinueCancel( this, i18n("There seems to be not enough free space in temporary directory. "
+							 "Write anyway?") ) == KMessageBox::Cancel )
+	return;
+    }
+  }
+
+
   K3bJobProgressDialog* dlg = 0;
   if( m_checkOnlyCreateImage->isChecked() ) {
     dlg = new K3bJobProgressDialog( kapp->mainWidget() );
@@ -277,18 +294,18 @@ void K3bCdCopyDialog::slotStartClicked()
     // m_tempDirSelectionWidget-tempPath() + ".toc"
     //
     if( QFileInfo( m_tempDirSelectionWidget->tempPath() ).isFile() ) {
-      if( KMessageBox::warningYesNo( this,
+      if( KMessageBox::warningContinueCancel( this,
 				     i18n("Do you want to overwrite %1?").arg(m_tempDirSelectionWidget->tempPath()),
-				     i18n("File Exists") )
-	  != KMessageBox::Yes )
+				     i18n("File Exists"), i18n("Overwrite") )
+	  != KMessageBox::Continue )
 	return;
     }
     
     if( QFileInfo( m_tempDirSelectionWidget->tempPath() + ".toc" ).isFile() ) {
-      if( KMessageBox::warningYesNo( this,
+      if( KMessageBox::warningContinueCancel( this,
 				     i18n("Do you want to overwrite %1?").arg(m_tempDirSelectionWidget->tempPath() + ".toc"),
-				     i18n("File Exists") )
-	  != KMessageBox::Yes )
+				     i18n("File Exists"), i18n("Overwrite") )
+	  != KMessageBox::Continue )
 	return;
     }
     
@@ -298,12 +315,12 @@ void K3bCdCopyDialog::slotStartClicked()
     job->setReaderDevice( m_comboSourceDevice->selectedDevice() );
     job->setImagePath( m_tempDirSelectionWidget->tempPath() );
     job->setNoCorrection( m_checkNoCorrection->isChecked() );
-    job->setRemoveImageFiles( m_checkDeleteImages->isChecked() );
+    job->setRemoveImageFiles( m_checkDeleteImages->isChecked() && !m_checkOnlyCreateImage->isChecked() );
     job->setOnlyCreateImage( m_checkOnlyCreateImage->isChecked() );
     job->setSimulate( m_checkSimulate->isChecked() );
     job->setWriteSpeed( m_writerSelectionWidget->writerSpeed() );
     job->setCopies( m_checkSimulate->isChecked() ? 1 : m_spinCopies->value() );
-    job->setReadRetries( m_spinRetries->value() );
+    job->setReadRetries( m_spinDataRetries->value() );
 
     burnJob = job;
   }
@@ -314,81 +331,125 @@ void K3bCdCopyDialog::slotStartClicked()
     job->setReaderDevice( m_comboSourceDevice->selectedDevice() );
     job->setSpeed( m_writerSelectionWidget->writerSpeed() );
     job->setSimulate( m_checkSimulate->isChecked() );
-    job->setOnTheFly( m_checkOnTheFly->isChecked() );
-    job->setKeepImage( !m_checkDeleteImages->isChecked() );
+    job->setOnTheFly( !m_checkCacheImage->isChecked() );
+    job->setKeepImage( !m_checkDeleteImages->isChecked() || m_checkOnlyCreateImage->isChecked() );
     job->setOnlyCreateImage( m_checkOnlyCreateImage->isChecked() );
     job->setTempPath( m_tempDirSelectionWidget->plainTempPath() );
     job->setCopies( m_checkSimulate->isChecked() ? 1 : m_spinCopies->value() );
     job->setParanoiaMode( m_comboParanoiaMode->currentText().toInt() );
-    job->setReadRetries( m_spinRetries->value() );
+    job->setDataReadRetries( m_spinDataRetries->value() );
+    job->setAudioReadRetries( m_spinAudioRetries->value() );
     job->setCopyCdText( m_checkReadCdText->isChecked() );
     job->setPreferCdText( m_checkPrefereCdText->isChecked() );
-    job->setIgnoreReadErrors( m_checkIgnoreReadErrors->isChecked() );
+    job->setIgnoreDataReadErrors( m_checkIgnoreDataReadErrors->isChecked() );
+    job->setIgnoreAudioReadErrors( m_checkIgnoreAudioReadErrors->isChecked() );
     job->setNoCorrection( m_checkNoCorrection->isChecked() );
     job->setWritingMode( m_writingModeWidget->writingMode() );
 
     burnJob = job;
   }
 
-  
-  hide();
+  if( !exitLoopOnHide() )
+    hide();
+
   dlg->startJob( burnJob );
-  show();
+
   delete dlg;
   delete burnJob;
+
+  if( KConfigGroup( k3bcore->config(), "General Options" ).readBoolEntry( "keep action dialogs open", false ) &&
+      !exitLoopOnHide() )
+    show();
+  else
+    close();
 }
 
 
 void K3bCdCopyDialog::slotToggleAll()
 {
+  updateOverrideDevice();
+
   K3bDevice::Device* dev = m_writerSelectionWidget->writerDevice();
 
   m_checkSimulate->setEnabled( !m_checkOnlyCreateImage->isChecked() );
-  m_checkDeleteImages->setEnabled( !m_checkOnlyCreateImage->isChecked() && !m_checkOnTheFly->isChecked() );
+  m_checkDeleteImages->setEnabled( !m_checkOnlyCreateImage->isChecked() && m_checkCacheImage->isChecked() );
   m_spinCopies->setDisabled( m_checkSimulate->isChecked() || m_checkOnlyCreateImage->isChecked() );
-  m_tempDirSelectionWidget->setDisabled( m_checkOnTheFly->isChecked() );
-  m_checkOnlyCreateImage->setEnabled( !m_checkOnTheFly->isChecked() );
+  m_tempDirSelectionWidget->setDisabled( !m_checkCacheImage->isChecked() );
+  m_checkOnlyCreateImage->setEnabled( m_checkCacheImage->isChecked() );
   m_writerSelectionWidget->setDisabled( m_checkOnlyCreateImage->isChecked() );
-  m_checkOnTheFly->setEnabled( !m_checkOnlyCreateImage->isChecked() );
+  m_checkCacheImage->setEnabled( !m_checkOnlyCreateImage->isChecked() );
 
-  if ( m_checkOnlyCreateImage->isChecked() )
-    m_checkDeleteImages->setChecked( false );
+  if( m_comboCopyMode->currentItem() == 1 ) {
+    // cdrecord does not support cloning on-the-fly
+    m_checkCacheImage->setChecked(true);
+    m_checkCacheImage->setEnabled(false);
+    
+    m_writingModeWidget->setSupportedModes( K3b::RAW );
+  }
+  else {
+    m_writingModeWidget->setSupportedModes( K3b::TAO|K3b::DAO|K3b::RAW );
+  }
 
-   if( m_comboCopyMode->currentItem() == 1 ) {
-     // cdrecord does not support cloning on-the-fly
-     m_checkOnTheFly->setChecked(false);
-     m_checkOnTheFly->setEnabled(false);
+  
+  
+  static_cast<QWidget*>( child( "audio_options" ) )->setDisabled( m_comboCopyMode->currentItem() == 1 );
+  
+  m_checkIgnoreDataReadErrors->setDisabled( m_comboCopyMode->currentItem() == 1 );
+  
+  m_groupAdvancedAudioOptions->setEnabled( k3bappcore->mediaCache()->medium( m_comboSourceDevice->selectedDevice() ).content() & K3bMedium::CONTENT_AUDIO &&
+					   m_comboCopyMode->currentItem() == 0 );
+  
+  m_writingModeWidget->setEnabled( !m_checkOnlyCreateImage->isChecked() );
 
-     m_writingModeWidget->setSupportedModes( K3b::RAW );
-   }
-   else {
-     if( dev == m_comboSourceDevice->selectedDevice() ) {
-       m_checkOnTheFly->setChecked( false );
-       m_checkOnTheFly->setEnabled( false );
-     }
-     else
-       m_checkOnTheFly->setEnabled( !m_checkOnlyCreateImage->isChecked() );
+  m_tempDirSelectionWidget->setNeededSize( neededSize() );
+  
+  setButtonEnabled( START_BUTTON, m_comboSourceDevice->selectedDevice() && 
+		    (dev || m_checkOnlyCreateImage->isChecked()) );
+}
 
-     m_writingModeWidget->setSupportedModes( K3b::TAO|K3b::DAO|K3b::RAW );
-   }
 
-   m_comboParanoiaMode->setDisabled( m_comboCopyMode->currentItem() == 1 );
+void K3bCdCopyDialog::slotSourceMediumChanged( K3bDevice::Device* dev )
+{
+  updateOverrideDevice();
 
-   // no CD-TEXT in TAO mode
-   m_checkReadCdText->setDisabled( m_writingModeWidget->writingMode() == K3b::TAO ||
-				   m_comboCopyMode->currentItem() == 1 );
-   m_checkPrefereCdText->setDisabled( !m_checkReadCdText->isChecked() || 
-				      m_writingModeWidget->writingMode() == K3b::TAO ||
-				      m_comboCopyMode->currentItem() == 1 );
+  K3bMedium medium = k3bappcore->mediaCache()->medium( dev );
 
-   m_checkIgnoreReadErrors->setDisabled( m_comboCopyMode->currentItem() == 1 );
+  m_tempDirSelectionWidget->setNeededSize( neededSize() );
 
-   //   m_checkNoCorrection->setEnabled( m_comboCopyMode->currentItem() == 1 );
+  if( k3bappcore->mediaCache()->toc( dev ).contentType() == K3bDevice::DATA ) {
+    m_tempDirSelectionWidget->setSelectionMode( K3bTempDirSelectionWidget::FILE );
+    m_tempDirSelectionWidget->setTempPath( m_tempDirSelectionWidget->tempDirectory() 
+					   + medium.volumeId().lower() 
+					   + ( medium.toc().count() == 1 ? QString(".iso") : QString::null ) );
+  }
+  else {
+    m_tempDirSelectionWidget->setSelectionMode( K3bTempDirSelectionWidget::DIR );
+  }
 
-   m_writingModeWidget->setEnabled( !m_checkOnlyCreateImage->isChecked() );
+  m_groupAdvancedAudioOptions->setEnabled( k3bappcore->mediaCache()->medium( dev ).content() & K3bMedium::CONTENT_AUDIO );
+  m_groupAdvancedDataOptions->setEnabled( k3bappcore->mediaCache()->medium( dev ).content() & K3bMedium::CONTENT_DATA );
 
-   m_buttonStart->setEnabled( m_comboSourceDevice->selectedDevice() && 
-			      (dev || m_checkOnlyCreateImage->isChecked()) );
+  // we can only clone single session CDs
+  if( k3bappcore->mediaCache()->toc( dev ).sessions() > 1 ) {
+    m_comboCopyMode->setEnabled( false );
+    m_comboCopyMode->setCurrentItem( 0 );
+  }
+  else {
+    m_comboCopyMode->setEnabled( true );
+  }
+
+  toggleAll();
+}
+
+
+void K3bCdCopyDialog::updateOverrideDevice()
+{
+  if( !m_checkCacheImage->isChecked() )
+    m_writerSelectionWidget->setOverrideDevice( 0 );
+  else
+    m_writerSelectionWidget->setOverrideDevice( m_comboSourceDevice->selectedDevice(),
+						i18n("Use the same device for burning"),
+						i18n("<qt>Use the same device for burning <i>(Or insert another medium)</i>") );
 }
 
 
@@ -398,14 +459,14 @@ void K3bCdCopyDialog::loadUserDefaults( KConfigBase* c )
   m_comboSourceDevice->setSelectedDevice( k3bcore->deviceManager()->findDevice( c->readEntry( "source_device" ) ) );
   m_writingModeWidget->loadConfig( c );
   m_checkSimulate->setChecked( c->readBoolEntry( "simulate", false ) );
-  m_checkOnTheFly->setChecked( c->readBoolEntry( "on_the_fly", false ) );
+  m_checkCacheImage->setChecked( !c->readBoolEntry( "on_the_fly", false ) );
   m_checkDeleteImages->setChecked( c->readBoolEntry( "delete_images", true ) );
   m_checkOnlyCreateImage->setChecked( c->readBoolEntry( "only_create_image", false ) );
   m_comboParanoiaMode->setCurrentItem( c->readNumEntry( "paranoia_mode", 0 ) );
 
   m_spinCopies->setValue( c->readNumEntry( "copies", 1 ) );
 
-  m_tempDirSelectionWidget->setTempPath( K3b::defaultTempPath() );
+  m_tempDirSelectionWidget->readConfig( c );
 
   if( c->readEntry( "copy mode", "normal" ) == "normal" )
     m_comboCopyMode->setCurrentItem( 0 );
@@ -414,36 +475,39 @@ void K3bCdCopyDialog::loadUserDefaults( KConfigBase* c )
 
   m_checkReadCdText->setChecked( c->readBoolEntry( "copy cdtext", true ) );
   m_checkPrefereCdText->setChecked( c->readBoolEntry( "prefer cdtext", false ) );
-  m_checkIgnoreReadErrors->setChecked( c->readBoolEntry( "ignore read errors", false ) );
+  m_checkIgnoreDataReadErrors->setChecked( c->readBoolEntry( "ignore data read errors", false ) );
+  m_checkIgnoreAudioReadErrors->setChecked( c->readBoolEntry( "ignore audio read errors", true ) );
   m_checkNoCorrection->setChecked( c->readBoolEntry( "no correction", false ) );
 
-  m_spinRetries->setValue( c->readNumEntry( "retries", 128 ) );
+  m_spinDataRetries->setValue( c->readNumEntry( "data retries", 128 ) );
+  m_spinAudioRetries->setValue( c->readNumEntry( "audio retries", 5 ) );
 
   slotToggleAll();
 }
 
+
 void K3bCdCopyDialog::saveUserDefaults( KConfigBase* c )
 {
   m_writingModeWidget->saveConfig( c );
   c->writeEntry( "simulate", m_checkSimulate->isChecked() );
-  c->writeEntry( "on_the_fly", m_checkOnTheFly->isChecked() );
+  c->writeEntry( "on_the_fly", !m_checkCacheImage->isChecked() );
   c->writeEntry( "delete_images", m_checkDeleteImages->isChecked() );
   c->writeEntry( "only_create_image", m_checkOnlyCreateImage->isChecked() );
   c->writeEntry( "paranoia_mode", m_comboParanoiaMode->currentText().toInt() );
   c->writeEntry( "copies", m_spinCopies->value() );
 
   m_writerSelectionWidget->saveConfig( c );
-
-  if( m_tempDirSelectionWidget->isEnabled() )
-    m_tempDirSelectionWidget->saveConfig();
+  m_tempDirSelectionWidget->saveConfig( c );
 
   c->writeEntry( "source_device", m_comboSourceDevice->selectedDevice()->devicename() );
 
   c->writeEntry( "copy cdtext", m_checkReadCdText->isChecked() );
   c->writeEntry( "prefer cdtext", m_checkPrefereCdText->isChecked() );
-  c->writeEntry( "ignore read errors", m_checkIgnoreReadErrors->isChecked() );
+  c->writeEntry( "ignore data read errors", m_checkIgnoreDataReadErrors->isChecked() );
+  c->writeEntry( "ignore audio read errors", m_checkIgnoreAudioReadErrors->isChecked() );
   c->writeEntry( "no correction", m_checkNoCorrection->isChecked() );
-  c->writeEntry( "retries", m_spinRetries->value() );
+  c->writeEntry( "data retries", m_spinDataRetries->value() );
+  c->writeEntry( "audio retries", m_spinAudioRetries->value() );
 
   QString s;
   if( m_comboCopyMode->currentItem() == 1 )
@@ -453,30 +517,37 @@ void K3bCdCopyDialog::saveUserDefaults( KConfigBase* c )
   c->writeEntry( "copy mode", s );
 }
 
+
 void K3bCdCopyDialog::loadK3bDefaults()
 {
   m_writingModeWidget->setWritingMode( K3b::WRITING_MODE_AUTO );
   m_writerSelectionWidget->loadDefaults();
   m_checkSimulate->setChecked( false );
-  m_checkOnTheFly->setChecked( false );
+  m_checkCacheImage->setChecked( true );
   m_checkDeleteImages->setChecked( true );
   m_checkOnlyCreateImage->setChecked( false );
   m_comboParanoiaMode->setCurrentItem(0);
   m_spinCopies->setValue(1);
   m_checkReadCdText->setChecked(true);
   m_checkPrefereCdText->setChecked(false);
-  m_checkIgnoreReadErrors->setChecked(false);
+  m_checkIgnoreDataReadErrors->setChecked(false);
+  m_checkIgnoreAudioReadErrors->setChecked(true);
   m_checkNoCorrection->setChecked(false);
   m_comboCopyMode->setCurrentItem( 0 ); // normal
-  m_spinRetries->setValue(128);
+  m_spinDataRetries->setValue(128);
+  m_spinAudioRetries->setValue(5);
+  m_tempDirSelectionWidget->setTempPath( K3b::defaultTempPath() );
 
   slotToggleAll();
 }
 
 
-void K3bCdCopyDialog::slotDeviceManagerChanged( K3bDevice::DeviceManager* dm )
+KIO::filesize_t K3bCdCopyDialog::neededSize() const
 {
-  m_comboSourceDevice->refreshDevices( dm->cdReader() );
+  if( m_comboCopyMode->currentItem() == 0 )
+    return k3bappcore->mediaCache()->medium( m_comboSourceDevice->selectedDevice() ).diskInfo().size().mode1Bytes();
+  else
+    return k3bappcore->mediaCache()->medium( m_comboSourceDevice->selectedDevice() ).diskInfo().size().rawBytes();
 }
 
 #include "k3bcdcopydialog.moc"
