@@ -1,10 +1,10 @@
 /* 
  *
- * $Id: k3baudiotrackview.cpp 509861 2006-02-15 18:35:29Z trueg $
+ * $Id: k3baudiotrackview.cpp 628165 2007-01-29 11:01:22Z trueg $
  * Copyright (C) 2004 Sebastian Trueg <trueg@k3b.org>
  *
  * This file is part of the K3b project.
- * Copyright (C) 1998-2004 Sebastian Trueg <trueg@k3b.org>
+ * Copyright (C) 1998-2007 Sebastian Trueg <trueg@k3b.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -29,11 +29,13 @@
 #include "k3baudiocdtrackdrag.h"
 #include "k3baudiocdtracksource.h"
 #include "k3baudiotracktrmlookupdialog.h"
+#include "k3baudiodatasourceeditwidget.h"
+#include "k3baudiotrackaddingdialog.h"
 
 #include <k3bview.h>
-#include <k3bcdtextvalidator.h>
 #include <k3blistviewitemanimator.h>
 #include <k3baudiodecoder.h>
+#include <k3bmsfedit.h>
 
 #include <qheader.h>
 #include <qtimer.h>
@@ -44,6 +46,9 @@
 #include <qevent.h>
 #include <qpixmap.h>
 #include <qpainter.h>
+#include <qlayout.h>
+#include <qlabel.h>
+#include <qvaluelist.h>
 
 #include <kurl.h>
 #include <kurldrag.h>
@@ -53,12 +58,14 @@
 #include <kiconloader.h>
 #include <kapplication.h>
 #include <kmessagebox.h>
+#include <kdialogbase.h>
 
 
 K3bAudioTrackView::K3bAudioTrackView( K3bAudioDoc* doc, QWidget* parent, const char* name )
   : K3bListView( parent, name ),
     m_doc(doc),
     m_updatingColumnWidths(false),
+    m_currentMouseOverItem(0),
     m_currentlyPlayingTrack(0)
 {
   m_player = new K3bAudioTrackPlayer( m_doc, this );
@@ -67,6 +74,7 @@ K3bAudioTrackView::K3bAudioTrackView( K3bAudioDoc* doc, QWidget* parent, const c
   connect( m_player, SIGNAL(paused(bool)), this, SLOT(togglePauseIndicator(bool)) );
   connect( m_player, SIGNAL(stopped()), this, SLOT(removePlayerIndicator()) );
 
+  setItemMargin( 5 );
   setAcceptDrops( true );
   setDropVisualizer( true );
   setAllColumnsShowFocus( true );
@@ -79,8 +87,6 @@ K3bAudioTrackView::K3bAudioTrackView( K3bAudioDoc* doc, QWidget* parent, const c
   setNoItemText( i18n("Use drag'n'drop to add audio files to the project.") + "\n"
 		 + i18n("After that press the burn button to write the CD." ) );
 
-  setValidator( new K3bCdTextValidator( this ) );
-
   setupColumns();
   setupActions();
 
@@ -107,6 +113,10 @@ K3bAudioTrackView::K3bAudioTrackView( K3bAudioDoc* doc, QWidget* parent, const c
 	   this, SLOT(slotTrackRemoved(K3bAudioTrack*)) );
 
   slotChanged();
+
+  // a little background pix hack because I am simply incapable of doing it another way. :(
+//   static QPixmap s_bgPix("/tmp/trueg/audio_bg.png");
+//   setK3bBackgroundPixmap( s_bgPix, TOP_LEFT );
 }
 
 
@@ -151,7 +161,7 @@ void K3bAudioTrackView::setupActions()
 				Key_Delete, this, SLOT(slotRemove()), 
 				actionCollection(), "track_remove" );
 
-  m_actionAddSilence = new KAction( i18n("Add Silence"), "misc",
+  m_actionAddSilence = new KAction( i18n("Add Silence") + "...", "misc",
 				    KShortcut(), this, SLOT(slotAddSilence()),
 				    actionCollection(), "track_add_silence" );
   m_actionMergeTracks = new KAction( i18n("Merge Tracks"), "misc",
@@ -163,6 +173,9 @@ void K3bAudioTrackView::setupActions()
   m_actionSplitTrack = new KAction( i18n("Split Track..."), 0,
 				    KShortcut(), this, SLOT(slotSplitTrack()),
 				    actionCollection(), "track_split" );
+  m_actionEditSource = new KAction( i18n("Edit Source..."), 0,
+				    KShortcut(), this, SLOT(slotEditSource()),
+				    actionCollection(), "source_edit" );
   m_actionPlayTrack = new KAction( i18n("Play Track"), "player_play",
 				   KShortcut(), this, SLOT(slotPlayTrack()),
 				   actionCollection(), "track_play" );
@@ -224,30 +237,30 @@ void K3bAudioTrackView::slotDropped( QDropEvent* e, QListViewItem* parent, QList
   if( !e->isAccepted() )
     return;
 
-  K3bAudioTrack* trackAfter = 0;
-  K3bAudioTrack* trackParent = 0;
-  K3bAudioDataSource* sourceAfter = 0;
+  m_dropTrackAfter = 0;
+  m_dropTrackParent = 0;
+  m_dropSourceAfter = 0;
   if( after ) {
     if( K3bAudioTrackViewItem* tv = dynamic_cast<K3bAudioTrackViewItem*>( after ) ) {
-      trackAfter = tv->track();
+      m_dropTrackAfter = tv->track();
     }
     else if( K3bAudioDataSourceViewItem* sv = dynamic_cast<K3bAudioDataSourceViewItem*>( after ) ) {
-      sourceAfter = sv->source();
+      m_dropSourceAfter = sv->source();
     }
   }
 
   if( K3bAudioTrackViewItem* tv = dynamic_cast<K3bAudioTrackViewItem*>( parent ) ) {
-    trackParent = tv->track();
+    m_dropTrackParent = tv->track();
   }
 
   //
   // In case the sources are not shown we do not want to handle them because the average
   // user would be confused otherwise
   //
-  if( trackParent && !m_trackItemMap[trackParent]->showingSources() ) {
+  if( m_dropTrackParent && !m_trackItemMap[m_dropTrackParent]->showingSources() ) {
     kdDebug() << "(K3bAudioTrackView) dropped after track which does not show it's sources." << endl;
-    trackAfter = trackParent;
-    trackParent = 0;
+    m_dropTrackAfter = m_dropTrackParent;
+    m_dropTrackParent = 0;
   }
 
   if( e->source() == viewport() ) {
@@ -278,14 +291,14 @@ void K3bAudioTrackView::slotDropped( QDropEvent* e, QListViewItem* parent, QList
     //
     for( QPtrListIterator<K3bAudioTrack> it( tracks ); it.current(); ++it ) {
       K3bAudioTrack* track = *it;
-      if( trackParent ) {
-	trackParent->merge( copyItems ? track->copy() : track, sourceAfter );
+      if( m_dropTrackParent ) {
+	m_dropTrackParent->merge( copyItems ? track->copy() : track, m_dropSourceAfter );
       }
-      else if( trackAfter ) {
+      else if( m_dropTrackAfter ) {
 	if( copyItems )
-	  track->copy()->moveAfter( trackAfter );
+	  track->copy()->moveAfter( m_dropTrackAfter );
 	else
-	  track->moveAfter( trackAfter );
+	  track->moveAfter( m_dropTrackAfter );
       }
       else {
 	if( copyItems )
@@ -300,18 +313,18 @@ void K3bAudioTrackView::slotDropped( QDropEvent* e, QListViewItem* parent, QList
     //
     for( QPtrListIterator<K3bAudioDataSource> it( sources ); it.current(); ++it ) {
       K3bAudioDataSource* source = *it;
-      if( trackParent ) {
-	if( sourceAfter ) {
+      if( m_dropTrackParent ) {
+	if( m_dropSourceAfter ) {
 	  if( copyItems )
-	    source->copy()->moveAfter( sourceAfter );
+	    source->copy()->moveAfter( m_dropSourceAfter );
 	  else
-	    source->moveAfter( sourceAfter );
+	    source->moveAfter( m_dropSourceAfter );
 	}
 	else {
 	  if( copyItems )
-	    source->copy()->moveAhead( trackParent->firstSource() );
+	    source->copy()->moveAhead( m_dropTrackParent->firstSource() );
 	  else
-	    source->moveAhead( trackParent->firstSource() );
+	    source->moveAhead( m_dropTrackParent->firstSource() );
 	}
       }
       else {
@@ -320,21 +333,21 @@ void K3bAudioTrackView::slotDropped( QDropEvent* e, QListViewItem* parent, QList
 
 	// special case: the source we remove from the track is the last and the track
 	// will be deleted.
-	if( !copyItems && trackAfter == source->track() && trackAfter->numberSources() == 1 )
-	  trackAfter = trackAfter->prev();
+	if( !copyItems && m_dropTrackAfter == source->track() && m_dropTrackAfter->numberSources() == 1 )
+	  m_dropTrackAfter = m_dropTrackAfter->prev();
 
 	if( copyItems )
 	  track->addSource( source->copy() );
 	else
 	  track->addSource( source );
 
-	if( trackAfter ) {
-	  track->moveAfter( trackAfter );
-	  trackAfter = track;
+	if( m_dropTrackAfter ) {
+	  track->moveAfter( m_dropTrackAfter );
+	  m_dropTrackAfter = track;
 	}
 	else {
 	  track->moveAhead( m_doc->firstTrack() );
-	  trackAfter = track;
+	  m_dropTrackAfter = track;
 	}
       }
     }
@@ -354,45 +367,50 @@ void K3bAudioTrackView::slotDropped( QDropEvent* e, QListViewItem* parent, QList
       int trackNumber = *it;
 
       K3bAudioCdTrackSource* source = new K3bAudioCdTrackSource( toc, trackNumber, cddb, dev );
-      if( trackParent ) {
-	source->moveAfter( sourceAfter );
-	if( sourceAfter )
-	  sourceAfter = source;
+      if( m_dropTrackParent ) {
+	source->moveAfter( m_dropSourceAfter );
+	if( m_dropSourceAfter )
+	  m_dropSourceAfter = source;
       }
       else {
 	K3bAudioTrack* track = new K3bAudioTrack();
 	track->setPerformer( cddb.artists[trackNumber-1] );
 	track->setTitle( cddb.titles[trackNumber-1] );
 	track->addSource( source );
-	if( trackAfter )
-	  track->moveAfter( trackAfter );
+	if( m_dropTrackAfter )
+	  track->moveAfter( m_dropTrackAfter );
 	else
 	  m_doc->addTrack( track, 0 );
 
-	trackAfter = track;
+	m_dropTrackAfter = track;
       }
     }
   }
   else{
-    KURL::List urls;
-    KURLDrag::decode( e, urls );
-
-    if( trackParent ) {
-      m_doc->addSources( trackParent, urls, sourceAfter );
-    }
-    else {
-      // add as new tracks
-      m_doc->addTracks( urls, trackAfter ? trackAfter->trackNumber() : 0 );
+    m_dropUrls.clear();
+    if( KURLDrag::decode( e, m_dropUrls ) ) {
+      //
+      // This is a small (not to ugly) hack to circumvent problems with the
+      // event queues: the url adding dialog will be non-modal regardless of
+      // the settings in case we open it directly.
+      //
+      QTimer::singleShot( 0, this, SLOT(slotAddUrls()) );
     }
   }
-
+  
   showAllSources();
-
+  
   // now grab that focus
   setFocus();
 }
 
 
+void K3bAudioTrackView::slotAddUrls()
+{
+  K3bAudioTrackAddingDialog::addUrls( m_dropUrls, m_doc, m_dropTrackAfter, m_dropTrackParent, m_dropSourceAfter, this );
+}
+
+
 void K3bAudioTrackView::slotChanged()
 {
   kdDebug() << "(K3bAudioTrackView::slotChanged)" << endl;
@@ -413,6 +431,8 @@ void K3bAudioTrackView::slotChanged()
     showAllSources();
   }
 
+  header()->setShown( m_doc->numOfTracks() > 0 );
+
   kdDebug() << "(K3bAudioTrackView::slotChanged) finished" << endl;
 }
 
@@ -736,12 +756,29 @@ void K3bAudioTrackView::slotAddSilence()
 {
   QListViewItem* item = selectedItems().first();
   if( item ) {
-    K3bAudioZeroData* zero = new K3bAudioZeroData();
-    if( K3bAudioTrackViewItem* tv = dynamic_cast<K3bAudioTrackViewItem*>(item) ) {
-      tv->track()->addSource( zero );
-    }
-    else if( K3bAudioDataSourceViewItem* sv = dynamic_cast<K3bAudioDataSourceViewItem*>(item) ) {
-      zero->moveAfter( sv->source() );
+    //
+    // create a simple dialog for asking the length of the silence
+    //
+    KDialogBase dlg( KDialogBase::Plain, 
+		     i18n("Add Silence"),
+		     KDialogBase::Ok|KDialogBase::Cancel,
+		     KDialogBase::Ok,
+		     this );
+    QHBoxLayout* dlgLayout = new QHBoxLayout( dlg.plainPage(), 0, KDialog::spacingHint() );
+    dlgLayout->setAutoAdd( true );
+    (void)new QLabel( i18n("Length of silence:"), dlg.plainPage() );
+    K3bMsfEdit* msfEdit = new K3bMsfEdit( dlg.plainPage() );
+    msfEdit->setValue( 150 ); // 2 seconds default
+    msfEdit->setFocus();
+
+    if( dlg.exec() == QDialog::Accepted ) {
+      K3bAudioZeroData* zero = new K3bAudioZeroData( msfEdit->value() );
+      if( K3bAudioTrackViewItem* tv = dynamic_cast<K3bAudioTrackViewItem*>(item) ) {
+	tv->track()->addSource( zero );
+      }
+      else if( K3bAudioDataSourceViewItem* sv = dynamic_cast<K3bAudioDataSourceViewItem*>(item) ) {
+	zero->moveAfter( sv->source() );
+      }
     }
   }
 }
@@ -776,6 +813,7 @@ void K3bAudioTrackView::slotSplitSource()
     track->moveAfter( trackAfter );
 
     // let's see if it's a file because in that case we can reuse the metainfo :)
+    // TODO: maybe add meta data to sources
     if( K3bAudioFile* file = dynamic_cast<K3bAudioFile*>( track->firstSource() ) ) {
       track->setArtist( file->decoder()->metaInfo( K3bAudioDecoder::META_ARTIST ) );
       track->setTitle( file->decoder()->metaInfo( K3bAudioDecoder::META_TITLE ) );
@@ -791,11 +829,38 @@ void K3bAudioTrackView::slotSplitTrack()
 {
   QListViewItem* item = selectedItems().first();
   if( K3bAudioTrackViewItem* tv = dynamic_cast<K3bAudioTrackViewItem*>(item) ) {
-    K3b::Msf pos;
-    if( K3bAudioTrackSplitDialog::getSplitPos( tv->track(), pos, this ) ) {
-      // TODO: should we fix the position to make sure tracks are at least 4 seconds long
-      tv->track()->split( pos );
-    }
+    K3bAudioTrackSplitDialog::splitTrack( tv->track(), this );
+  }
+}
+
+
+void K3bAudioTrackView::slotEditSource()
+{
+  QListViewItem* item = selectedItems().first();
+
+  K3bAudioDataSource* source = 0;
+  if( K3bAudioDataSourceViewItem* sv = dynamic_cast<K3bAudioDataSourceViewItem*>(item) )
+    source = sv->source();
+  else if( K3bAudioTrackViewItem* tv = dynamic_cast<K3bAudioTrackViewItem*>(item) )
+    source = tv->track()->firstSource();
+
+  if( source ) {
+    KDialogBase dlg( KDialogBase::Plain,
+		     i18n("Edit Audio Track Source"),
+		     KDialogBase::Ok|KDialogBase::Cancel,
+		     KDialogBase::Ok,
+		     this,
+		     0,
+		     true,
+		     true );
+    QVBoxLayout* lay = new QVBoxLayout( dlg.plainPage() );
+    lay->setMargin( 0 );
+    lay->setSpacing( KDialog::spacingHint() );
+    lay->setAutoAdd( true );
+    K3bAudioDataSourceEditWidget* editW = new K3bAudioDataSourceEditWidget( dlg.plainPage() );
+    editW->loadSource( source );
+    if( dlg.exec() == QDialog::Accepted )
+      editW->saveSource();
   }
 }
 
@@ -826,10 +891,19 @@ void K3bAudioTrackView::showPopupMenu( KListView*, QListViewItem* item, const QP
   if( numSources == 1 && numTracks == 0 ) {
     m_popupMenu->insertSeparator();
     m_actionSplitSource->plug( m_popupMenu );
+    m_actionEditSource->plug( m_popupMenu );
   }
   else if( numTracks == 1 && numSources == 0 ) {
     m_popupMenu->insertSeparator();
-    m_actionSplitTrack->plug( m_popupMenu );
+   
+    
+  
+    if( K3bAudioTrackViewItem* tv = dynamic_cast<K3bAudioTrackViewItem*>(item) )
+       if( tv->track()->length().lba() > 60 )
+          m_actionSplitTrack->plug( m_popupMenu );
+    
+    m_actionEditSource->plug( m_popupMenu );
+ 
   }
   else if( numTracks > 1 ) {
     m_popupMenu->insertSeparator();
@@ -854,8 +928,8 @@ void K3bAudioTrackView::slotProperties()
 
   if( !tracks.isEmpty() ) {
     K3bAudioTrackDialog d( tracks, this );
-    d.exec();
-  }
+     d.exec();
+  }  
   else {
     static_cast<K3bView*>(m_doc->view())->slotProperties();
   }
